#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from uds.uds_config_tool import DecodeFunctions
import sys
from uds.uds_config_tool.FunctionCreation.iServiceMethodFactory import IServiceMethodFactory


# Note: the request is not the simplest to parse from the ODX, so paritally hardcoding this one again (for now at least)
requestFuncTemplate = str("def {0}(DTCStatusMask=[],DTCMaskRecord=[],DTCSnapshotRecordNumber=[],DTCExtendedRecordNumber=[],DTCSeverityMask=[]):\n"
                          "    encoded = []\n"
                          "    {3}\n"
                          "    return {1} + {2} + encoded # ... SID, sub-func, and params")

checkFunctionTemplate = str("def {0}(input):\n"
                            "    serviceIdExpected = {1}\n"
                            "    subFunctionExpected = {2}\n"
                            "    serviceId = DecodeFunctions.buildIntFromList(input[{3}:{4}])\n"
                            "    subFunction = DecodeFunctions.buildIntFromList(input[{5}:{6}])\n"
                            "    if(serviceId != serviceIdExpected): raise Exception(\"Service Id Received not expected. Expected {{0}}; Got {{1}} \".format(serviceIdExpected, serviceId))\n"
                            "    if(subFunction != subFunctionExpected): raise Exception(\"Sub-function Received not expected. Expected {{0}}; Got {{1}} \".format(subFunctionExpected, subFunction))\n"
                            "{7}")

negativeResponseFuncTemplate = str("def {0}(input):\n"
                                   "    {1}")

encodePositiveResponseFuncTemplate = str("def {0}(input):\n"
                                         "    encoded = []\n"
                                         "    retval = None\n"
                                         "{1}\n"
                                         "    return retval")
							
							
class ReadDTCMethodFactory(IServiceMethodFactory):

    ##
    # @brief method to create the request function for the service element
    @staticmethod
    def create_requestFunction(diagServiceElement, xmlElements):
        # Note: due to the compleixty of the call, this one is partially hardcoded - we do at least check the ODX file far enough to ensure that the request and subfunction are accurate.
        serviceId = 0
        diagnosticId = 0

        shortName = "request_{0}".format(diagServiceElement.find('SHORT-NAME').text)
        requestElement = xmlElements[diagServiceElement.find('REQUEST-REF').attrib['ID-REF']]
        paramsElement = requestElement.find('PARAMS')
        encodeString = ""

        for param in paramsElement:
            semantic = None
            try:
                semantic = param.attrib['SEMANTIC']
            except AttributeError:
                pass
            except KeyError:
                pass

            if(semantic == 'SERVICE-ID'):
                serviceId = [int(param.find('CODED-VALUE').text)]
            elif(semantic == 'SUBFUNCTION'):
                shortName += param.find('SHORT-NAME').text
                subfunction = DecodeFunctions.intArrayToIntArray([int(param.find('CODED-VALUE').text)], 'int8', 'int8')
                if subfunction[0] in [0x01,0x02, 0x0F, 0x11, 0x12, 0x13]: # ... DTCStatusMask required for these subfunctions
                    encodeString = "encoded += DTCStatusMask"
                elif subfunction[0] in [0x03,0x04, 0x06, 0x09, 0x10]: # ... DTCMaskRecord required for these subfunctions
                    encodeString = "encoded += DTCMaskRecord # ... format is [0xNN,0xNN,0xNN]"
                elif subfunction[0] in [0x03,0x04, 0x05]: # ... DTCSnapshotRecordNumber required for these subfunctions
                    encodeString = "encoded += DTCSnapshotRecordNumber"
                elif subfunction[0] in [0x06,0x10]: # ... DTCExtendedRecordNumber required for these subfunctions
                    encodeString = "encoded += DTCExtendedRecordNumber"
                elif subfunction[0] in [0x07,0x08]: # ... DTCSeverityMaskRecord required for these subfunctions
                    encodeString = "encoded += DTCSeverityMask+DTCStatusMask"

        funcString = requestFuncTemplate.format(shortName,
                                                serviceId,
                                                subfunction,
                                                encodeString)
        exec(funcString)
        return (locals()[shortName],str(subfunction))

    ##
    # @brief method to create the function to check the positive response for validity
    @staticmethod
    def create_checkPositiveResponseFunction(diagServiceElement, xmlElements):
        responseId = 0
        subfunction = 0

        responseIdStart = 0
        responseIdEnd = 0
        subfunctionStart = 0
        subfunctionEnd = 0

        shortName = diagServiceElement.find('SHORT-NAME').text
        checkFunctionName = "check_{0}".format(shortName)
        positiveResponseElement = xmlElements[(diagServiceElement.find('POS-RESPONSE-REFS')).find('POS-RESPONSE-REF').attrib['ID-REF']]

        paramsElement = positiveResponseElement.find('PARAMS')

        totalLength = 0
        subfunctionChecks = ""

        for param in paramsElement:
            try:
                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except AttributeError:
                    pass

                startByte = int(param.find('BYTE-POSITION').text)

                if(semantic == 'SERVICE-ID'):
                    responseId = int(param.find('CODED-VALUE').text)
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    responseIdStart = startByte
                    responseIdEnd = startByte + listLength
                    totalLength += listLength
                elif(semantic == 'SUBFUNCTION'):
                    subfunction = int(param.find('CODED-VALUE').text)
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    subfunctionStart = startByte
                    subfunctionEnd = startByte + listLength
                    totalLength += listLength

                    if subfunction in [0x02,0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x13]: # ... DTCStatusMask required for these subfunctions
                        subfunctionChecks += "    if len(input) < 3: raise Exception(\"Total length returned not as expected. Expected: greater than or equal to 3; Got {{0}}\".format(len(input)))\n"
                        subfunctionChecks += "    if (len(input)-3)%4 != 0: raise Exception(\"Total length returned not as expected. Received a partial DTC and Status Record; Got {{0}} total length\".format(len(input)))\n"
                    elif subfunction in [0x01, 0x07, 0x11, 0x12]: # ... DTCStatusMask required for these subfunctions
                        subfunctionChecks += "    if len(input) != 6: raise Exception(\"Total length returned not as expected. Expected: 6; Got {{0}}\".format(len(input)))\n"
                    elif subfunction in [0x03]: # ... DTCStatusMask required for these subfunctions
                        subfunctionChecks += "    if len(input) < 2: raise Exception(\"Total length returned not as expected. Expected: greater than or equal to 2; Got {{0}}\".format(len(input)))\n"
                        subfunctionChecks += "    if (len(input)-2)%4 != 0: raise Exception(\"Total length returned not as expected. Received a partial DTC and Snapshot Record Number; Got {{0}} total length\".format(len(input)))\n"
                    elif subfunction in [0x04]: # ... DTCStatusMask required for these subfunctions
                        subfunctionChecks += "    pass #??? ... we need to parse the ODX for DTC length detials or this one, so leaving till spoken to Richard ???\n"
                    elif subfunction in [0x05]: # ... DTCStatusMask required for these subfunctions
                        subfunctionChecks += "    pass #??? ... we need to parse the ODX for DTC length detials or this one, so leaving till spoken to Richard ???\n"
                    elif subfunction in [0x06, 0x10]: # ... DTCStatusMask required for these subfunctions
                        subfunctionChecks += "    pass #??? ... we need to parse the ODX for DTC length detials or this one, so leaving till spoken to Richard ???\n"
                    elif subfunction in [0x08, 0x09]: # ... DTCStatusMask required for these subfunctions
                        subfunctionChecks += "    if len(input) < 3: raise Exception(\"Total length returned not as expected. Expected: greater than or equal to 3; Got {{0}}\".format(len(input)))\n"
                        subfunctionChecks += "    if (len(input)-3)%6 != 0: raise Exception(\"Total length returned not as expected. Received a partial DTC and Severity Record; Got {{0}} total length\".format(len(input)))\n"

                else:
                    pass
            except:
                #print(sys.exc_info())
                pass


        checkFunctionString = checkFunctionTemplate.format(checkFunctionName, # 0
                                                           responseId, # 1
                                                           subfunction, # 2
                                                           responseIdStart, # 3
                                                           responseIdEnd, # 4
                                                           subfunctionStart, # 5
                                                           subfunctionEnd, # 6
                                                           subfunctionChecks) # 7
        exec(checkFunctionString)
        return locals()[checkFunctionName]

    ##
    # @brief method to encode the positive response from the raw type to it physical representation
    @staticmethod
    def create_encodePositiveResponseFunction(diagServiceElement, xmlElements):
        # There's nothing to extract here! The only value in the response is the DID, checking of which is handled in the check function, 
        # so must be present and ok. This function is only required to return the default None response.
		
        shortName = diagServiceElement.find('SHORT-NAME').text
        encodePositiveResponseFunctionName = "encode_{0}".format(shortName)
        positiveResponseElement = xmlElements[(diagServiceElement.find('POS-RESPONSE-REFS')).find('POS-RESPONSE-REF').attrib['ID-REF']]
		
        paramsElement = positiveResponseElement.find('PARAMS')
        subfunctionResponse = ""

        for param in paramsElement:
            try:
                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except AttributeError:
                    pass

                if(semantic == 'SUBFUNCTION'):
                    subfunction = int(param.find('CODED-VALUE').text)
                    if subfunction in [0x02,0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x13]: # ... DTCStatusMask required for these subfunctions
                            subfunctionResponse += "    retval = {'DTCStatusAvailabilityMask':input[2:3], 'DTCAndStatusRecord':[]}\n"
                            subfunctionResponse += "    records = input[3:]\n"
                            subfunctionResponse += "    for i in range(int(len(records)/4)):\n"
                            subfunctionResponse += "        recStart = i*4\n"
                            subfunctionResponse += "        retval['DTCAndStatusRecord'].append({'DTC':records[recStart:recStart+3],'statusOfDTC':records[recStart+3:recStart+4]})\n"
                    elif subfunction in [0x01, 0x07, 0x11, 0x12]: # ... DTCStatusMask required for these subfunctions
                            subfunctionResponse += "    retval = {'DTCStatusAvailabilityMask':input[2:3], 'DTCFormatIdentifier':input[3:4], 'DTCCount':[(input[4]<<8)+input[5]]}  # ... DTCCount decoded as int16\n"
                    elif subfunction in [0x03]: # ... DTCStatusMask required for these subfunctions
                            subfunctionResponse += "    retval = []\n"
                            subfunctionResponse += "    records = input[3:]\n"
                            subfunctionResponse += "    for i in range(int(len(records)/4)):\n"
                            subfunctionResponse += "        recStart = i*4\n"
                            subfunctionResponse += "        retval.append({'DTC':records[recStart:recStart+3],'DTCSnapshotRecordNumber':records[recStart+3:recStart+4]})\n"
                    elif subfunction in [0x04]: # ... DTCStatusMask required for these subfunctions
                            subfunctionResponse += "    pass #??? ... we need to parse the ODX for DTC length detials or this one, so leaving till spoken to Richard ???\n"
                    elif subfunction in [0x05]: # ... DTCStatusMask required for these subfunctions
                            subfunctionResponse += "    pass #??? ... we need to parse the ODX for DTC length detials or this one, so leaving till spoken to Richard ???\n"
                    elif subfunction in [0x06, 0x10]: # ... DTCStatusMask required for these subfunctions
                            subfunctionResponse += "    pass #??? ... we need to parse the ODX for DTC length detials or this one, so leaving till spoken to Richard ???\n"
                    elif subfunction in [0x08, 0x09]: # ... DTCStatusMask required for these subfunctions
                            subfunctionResponse += "    retval = {'DTCStatusAvailabilityMask':input[2:3], 'DTCAndSeverityRecord':[]}\n"
                            subfunctionResponse += "    records = input[3:]\n"
                            subfunctionResponse += "    for i in range(int(len(records)/6)):\n"
                            subfunctionResponse += "        recStart = i*6\n"
                            subfunctionResponse += "        retval['DTCAndSeverityRecord'].append({'DTCSeverity':records[recStart:recStart+1],'DTCFunctionalUnit':records[recStart+1:recStart+2],'DTC':records[recStart+2:recStart+5],'statusOfDTC':records[recStart+5:recStart+6]})\n"
            except:
                pass


        encodeFunctionString = encodePositiveResponseFuncTemplate.format(encodePositiveResponseFunctionName, # 0
                                                                         subfunctionResponse) # 1
        exec(encodeFunctionString)
        return locals()[encodePositiveResponseFunctionName]


    ##
    # @brief method to create the negative response function for the service element
    @staticmethod
    def create_checkNegativeResponseFunction(diagServiceElement, xmlElements):
        shortName = diagServiceElement.find('SHORT-NAME').text
        check_negativeResponseFunctionName = "check_negResponse_{0}".format(shortName)

        negativeResponsesElement = diagServiceElement.find('NEG-RESPONSE-REFS')

        negativeResponseChecks = []

        for negativeResponse in negativeResponsesElement:
            negativeResponseRef = xmlElements[negativeResponse.attrib['ID-REF']]

            negativeResponseParams = negativeResponseRef.find('PARAMS')

            for param in negativeResponseParams:

                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except:
                    semantic = None

                if semantic == 'SERVICE-ID':
                    serviceId = param.find('CODED-VALUE').text
                    start = int(param.find('BYTE-POSITION').text)
                    diagCodedType = param.find('DIAG-CODED-TYPE')
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength/8)
                    end = start + listLength

                    checkString = "if input[{0}:{1}] == [{2}]: raise Exception(\"Detected negative response: {{0}}\".format(str([hex(n) for n in input])))".format(start,
                                                                                                                                                                   end,
                                                                                                                                                                   serviceId)
                    negativeResponseChecks.append(checkString)

                    pass
                pass

        negativeResponseFunctionString = negativeResponseFuncTemplate.format(check_negativeResponseFunctionName,
                                                                             "\n....".join(negativeResponseChecks))
        exec(negativeResponseFunctionString)
        return locals()[check_negativeResponseFunctionName]



import can
from can.interfaces import pcan, vector ,kvaser
import pdb
from uds_configuration.Config import Config
from os import path
#from uds import CanConnection
from can_connect.CanConnection import CanConnection

class CanConnectionFactory(object):

    connections = {}
    config = None

    @staticmethod
    def __call__(callback=None, filter=None, configPath=None, **kwargs):

        # pdb.set_trace()

        CanConnectionFactory.loadConfiguration(configPath)
        CanConnectionFactory.checkKwargs(**kwargs)

        # check config file and load
        connectionType = CanConnectionFactory.config['can']['interface']

        if connectionType == 'virtual':
            connectionName = CanConnectionFactory.config['virtual']['interfaceName']
            if connectionName not in CanConnectionFactory.connections:
                CanConnectionFactory.connections[connectionName] = CanConnection(callback, filter,
                                                                                 can.interface.Bus(connectionName,
                                                                                     bustype='virtual'))
            else:
                CanConnectionFactory.connections[connectionName].addCallback(callback)
                CanConnectionFactory.connections[connectionName].addFilter(filter)
            return CanConnectionFactory.connections[connectionName]

        elif connectionType == 'peak':
            channel = CanConnectionFactory.config['peak']['device']
            if channel not in CanConnectionFactory.connections:
                baudrate = CanConnectionFactory.config['can']['baudrate']
                CanConnectionFactory.connections[channel] = CanConnection(callback, filter,
                                                                          pcan.PcanBus(channel,
                                                                          bitrate=baudrate))
            else:
                CanConnectionFactory.connections[channel].addCallback(callback)
                CanConnectionFactory.connections[channel].addFilter(filter)
                
            return CanConnectionFactory.connections[channel]

        elif connectionType == 'vector':
            channel = int(CanConnectionFactory.config['vector']['channel'])
            app_name = CanConnectionFactory.config['vector']['appName']
            connectionKey = str("{0}_{1}").format(app_name, channel)
            if connectionKey not in CanConnectionFactory.connections:
                baudrate = int(CanConnectionFactory.config['can']['baudrate'])
                CanConnectionFactory.connections[connectionKey] = CanConnection(callback, filter,
                                                                                vector.VectorBus(channel,
                                                                                    app_name=app_name,
                                                                                    data_bitrate=baudrate))
            else:
                CanConnectionFactory.connections[connectionKey].addCallback(callback)
                CanConnectionFactory.connections[connectionKey].addFilter(filter)
            return CanConnectionFactory.connections[connectionKey]
    
        elif connectionType == 'kvaser':  # New Kvaser connection type
            channel = CanConnectionFactory.config['kvaser']['channel']
            if channel not in CanConnectionFactory.connections:
                baudrate = CanConnectionFactory.config['can']['baudrate']
                CanConnectionFactory.connections[channel] = CanConnection(callback, filter,
                                                                          kvaser.KvaserBus(channel,
                                                                                           bitrate=int(baudrate)))
            else:
                CanConnectionFactory.connections[channel].addCallback(callback)
                CanConnectionFactory.connections[channel].addFilter(filter)
            return CanConnectionFactory.connections[channel]

    @staticmethod
    def loadConfiguration(configPath=None):

        CanConnectionFactory.config = Config()

        localConfig = path.dirname(__file__) + "\config.ini"
        CanConnectionFactory.config.read(localConfig)

        if configPath is not None:
            if path.exists(configPath):
                CanConnectionFactory.config.read(configPath)
            else:
                raise FileNotFoundError("Can not find config file")

    # @staticmethod
    # def checkKwargs(**kwargs):

    #     if 'interface' in kwargs:
    #         CanConnectionFactory.config['can']['interface'] = kwargs['interface']

    #     if 'baudrate' in kwargs:
    #         CanConnectionFactory.config['can']['baudrate'] = kwargs['baudrate']

    #     if 'device' in kwargs:
    #         CanConnectionFactory.config['peak']['device'] = kwargs['device']

    #     if 'appName' in kwargs:
    #         CanConnectionFactory.config['vector']['appName'] = kwargs['appName']

    #     if 'channel' in kwargs:
    #         CanConnectionFactory.config['vector']['channel'] = kwargs['channel']

    @staticmethod
    def checkKwargs(**kwargs):

        if 'interface' in kwargs:
            CanConnectionFactory.config['can']['interface'] = str(kwargs['interface'])

        if 'baudrate' in kwargs:
            CanConnectionFactory.config['can']['baudrate'] = str(kwargs['baudrate'])

        if 'device' in kwargs:
            CanConnectionFactory.config['peak']['device'] = str(kwargs['device'])
            CanConnectionFactory.config['kvaser']['device'] = str(kwargs['device'])  # Assuming device is the same

        if 'appName' in kwargs:
            CanConnectionFactory.config['vector']['appName'] = str(kwargs['appName'])

        if 'channel' in kwargs:
            CanConnectionFactory.config['vector']['channel'] = str(kwargs['channel'])  # Convert to string for config
            CanConnectionFactory.config['kvaser']['channel'] = str(kwargs['channel'])  # Add channel for Kvaser if needed





#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from uds.uds_config_tool.SupportedServices.iContainer import iContainer
from types import MethodType


class SecurityAccessContainer(object):

    __metaclass__ = iContainer

    def __init__(self):
        self.requestFunctions = {}
        self.checkFunctions = {}
        self.negativeResponseFunctions = {}
        self.positiveResponseFunctions = {}

    @staticmethod
    def __securityAccess(target, parameter, key=None, suppressResponse=False):

        requestFunction = target.securityAccessContainer.requestFunctions[parameter]
        checkNegativeResponseFunction = target.securityAccessContainer.negativeResponseFunctions[parameter]
        checkPositiveResponseFunctions = target.securityAccessContainer.positiveResponseFunctions[parameter]
        checkSidFunction = checkPositiveResponseFunctions[0]
        checkSecurityAccessFunction = checkPositiveResponseFunctions[1]
        checkDataFunction = checkPositiveResponseFunctions[2]

        # if the key is not none then we are sending a key back to the ECU check the key type
        if key is not None:
            #check key format
            # send request for key response
            response = target.send(requestFunction(key, suppressResponse), responseRequired=not(suppressResponse))
        else:
            response = target.send(requestFunction(suppressResponse))

        if suppressResponse is False:
            checkNegativeResponseFunction(response)

        if checkDataFunction is None:
            output = None
        else:
            checkDataFunction(response[2:])
            output = response[2:]

        return output

    def bind_function(self, bindObject):
        bindObject.securityAccess = MethodType(self.__securityAccess, bindObject)

    def add_requestFunction(self, aFunction, dictionaryEntry):
        self.requestFunctions[dictionaryEntry] = aFunction

    def add_checkFunction(self, aFunction, dictionaryEntry):
        self.checkFunctions[dictionaryEntry] = aFunction

    def add_negativeResponseFunction(self, aFunction, dictionaryEntry):
        self.negativeResponseFunctions[dictionaryEntry] = aFunction

    def add_positiveResponseFunction(self, aFunction, dictionaryEntry):
        self.positiveResponseFunctions[dictionaryEntry] = aFunction



import can

class CanConnection(object):
    class CustomListener(can.Listener):  # Subclass Listener to handle messages
        def __init__(self, callback):
            super().__init__()
            self.callback = callback

        def on_message_received(self, msg):
            self.callback(msg)

    def __init__(self, callback, filter, bus):
        self.__bus = bus
        listener = self.CustomListener(callback)  # Use CustomListener instead of can.Listener
        self.__notifier = can.Notifier(self.__bus, [listener], 0)
        self.__listeners = [listener]
        self.addFilter(filter)

    def addCallback(self, callback):
        listener = self.CustomListener(callback)  # Use CustomListener for each callback
        self.__notifier.add_listener(listener)
        self.__listeners.append(listener)

    def addFilter(self, filter):
        filters = self.__bus.filters
        if filters is not None:
            filters.append({"can_id": filter, "can_mask": 0xFFF, "extended": False})
        else:
            filters = [{"can_id": filter, "can_mask": 0xFFF, "extended": False}]
        self.__bus.set_filters(filters)

    def transmit(self, data, reqId, extended=False):
        canMsg = can.Message(arbitration_id=reqId, is_extended_id=extended)
        canMsg.dlc = 8
        canMsg.data = data
        self.__bus.send(canMsg)



#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from uds.uds_config_tool.SupportedServices.iContainer import iContainer
from types import MethodType
import threading
import time


class TesterPresentContainer(object):

    __metaclass__ = iContainer
	
    testerPresentThreadRef = None
    testerPresentTargets = set()

    def __init__(self):
        self.requestFunctions = {}
        self.checkFunctions = {}
        self.negativeResponseFunctions = {}
        self.positiveResponseFunctions = {}

    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.testerPresentContainer() It does not operate
    # on this instance of the container class.
    @staticmethod
    def __testerPresent(target, suppressResponse=True, disable=False, **kwargs):

        # If the disable flag is set, we do nothing but remove any testerPresent behaviour for the current session
        if disable:
            target.testerPresentDisable() # ... see diagnostic session control container
            return

        # Note: testerPresentContainer has no DID required and only supports a zeroSubFunction in order to support response suppression.
        requestFunction = target.testerPresentContainer.requestFunctions["TesterPresent"]
        if "TesterPresent" in target.testerPresentContainer.checkFunctions:
            checkFunction = target.testerPresentContainer.checkFunctions["TesterPresent"]
        else:
            checkFunction = None
        negativeResponseFunction = target.testerPresentContainer.negativeResponseFunctions["TesterPresent"]
        if "TesterPresent" in target.testerPresentContainer.positiveResponseFunctions:
            positiveResponseFunction = target.testerPresentContainer.positiveResponseFunctions["TesterPresent"]
        else:
            positiveResponseFunction = None

        # Call the sequence of functions to execute the Tester Present request/response action ...
        # ==============================================================================

        if checkFunction is None or positiveResponseFunction is None:
            suppressResponse = True

        # Create the request ...
        request = requestFunction(suppressResponse)

        if suppressResponse == False:
            # Send request and receive the response ...
            response = target.send(request,responseRequired=True) # ... this returns a single response
            negativeResponseFunction(response)  # ... throws an exception to be handled at a higher level if a negative response is received

            # We have a positive response so check that it makes sense to us ...
            checkFunction(response)

            # All is still good, so return the response (currently this function does nothing, but including it here as a hook in case that changes) ...
            return positiveResponseFunction(response)
			
		# ... else ...
        # Send request and receive the response ...
        response = target.send(request,responseRequired=False) # ... this suppresses any response handling (not expected)
        return


    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.testerPresentThread() It does not operate
    # on this instance of the container class.
    # Important Note: we always keep a single thread running in the background monitoring the testerPresent requirements.
    # As this is static, and we can have many ECU connections via different UDS instances, this means we need to check them all!
    @staticmethod
    def __testerPresentThread(target, **kwargs):

        def __tpWorker():
            #print("work thread started (should be once only)")
            while True:
                #print("inside worker loop")
                for tgt in TesterPresentContainer.testerPresentTargets:
                    try:
                        transmitting = tgt.isTransmitting()
                    except:
                        continue  # ... there's a problem with the stored target - e.g. target no longer in use, so a dead reference - so skip it
                    # ... otherwise we continue outside of the try/except block to avoid trapping any exceptions that may need to be propagated upwards
                    #print("target found")
                    if not transmitting:
                        #print("target not transmitting")
                        tpSessionRecord = tgt.testerPresentSessionRecord()
                        if tpSessionRecord['reqd']: # ... testPresent behaviour is required for the current diagnostic session
                            #print("tp required for target")
                            if tgt.sessionTimeSinceLastSend() >= tpSessionRecord['timeout']:
                                #print("timed out! - sending test present")
                                tgt.testerPresent()
                if not threading.main_thread().is_alive():
                    return
                time.sleep(1.0) # ... check if tester present is required every 1s (we are unlikely to require finer granularity).
                # Note: I'm avoiding direct wait mechanisms (of testerPresent TO) to allow for radical difference in behaviour for changing diagnostic sessions.
                # This can of course be changed.

        TesterPresentContainer.testerPresentTargets.add(target) # ... track a list of all possible concurrent targets, as we process tester present for all targets via one thread
        if TesterPresentContainer.testerPresentThreadRef is None:
            TesterPresentContainer.testerPresentThreadRef = threading.Thread(name='tpWorker', target=__tpWorker)
            TesterPresentContainer.testerPresentThreadRef.start()


    def bind_function(self, bindObject):
        bindObject.testerPresent = MethodType(self.__testerPresent, bindObject)
        bindObject.testerPresentThread = MethodType(self.__testerPresentThread, bindObject)

    def add_requestFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.requestFunctions["TesterPresent"] = aFunction

    def add_checkFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.checkFunctions["TesterPresent"] = aFunction

    def add_negativeResponseFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.negativeResponseFunctions["TesterPresent"] = aFunction

    def add_positiveResponseFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.positiveResponseFunctions["TesterPresent"] = aFunction


#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from uds.uds_config_tool import DecodeFunctions
import sys
from uds.uds_config_tool.FunctionCreation.iServiceMethodFactory import IServiceMethodFactory


# When encode the dataRecord for transmission we have to allow for multiple elements in the data record
# i.e. 'value1' - for a single value, or [('param1','value1'),('param2','value2')]  for more complex data records
requestFuncTemplate = str("def {0}(groupOfDTC):\n"
                          "    {2}\n"
                          "    return {1} + encoded")									 


checkFunctionTemplate = str("def {0}(input):\n"
                            "    serviceIdExpected = {1}\n"
                            "    serviceId = DecodeFunctions.buildIntFromList(input[{2}:{3}])\n"
                            "    if(len(input) != {4}): raise Exception(\"Total length returned not as expected. Expected: {4}; Got {{0}}\".format(len(input)))\n"
                            "    if(serviceId != serviceIdExpected): raise Exception(\"Service Id Received not expected. Expected {{0}}; Got {{1}} \".format(serviceIdExpected, serviceId))")

negativeResponseFuncTemplate = str("def {0}(input):\n"
                                   "    {1}")

encodePositiveResponseFuncTemplate = str("def {0}(input):\n"
                                         "    return")

class ClearDTCMethodFactory(IServiceMethodFactory):

    ##
    # @brief method to create the request function for the service element
    @staticmethod
    def create_requestFunction(diagServiceElement, xmlElements):
        serviceId = 0
        diagnosticId = 0

        shortName = "request_{0}".format(diagServiceElement.find('SHORT-NAME').text)
        requestElement = xmlElements[diagServiceElement.find('REQUEST-REF').attrib['ID-REF']]
        paramsElement = requestElement.find('PARAMS')

        encodeFunction = "None"

        for param in paramsElement:
            semantic = None
            try:
                semantic = param.attrib['SEMANTIC']
            except AttributeError:
                pass

            if(semantic == 'SERVICE-ID'):
                serviceId = [int(param.find('CODED-VALUE').text)]
            elif(semantic == 'ID'):
                diagnosticId = DecodeFunctions.intArrayToIntArray([int(param.find('CODED-VALUE').text)], 'int16', 'int8')
            elif semantic == 'DATA':
                dataObjectElement = xmlElements[(param.find('DOP-REF')).attrib['ID-REF']]
                longName = param.find('LONG-NAME').text
                bytePosition = int(param.find('BYTE-POSITION').text)
                # We're not worrying about the formatting here, as we simply need the 3 byte hex to identify the DTC or DTC group ... (but can be changed if required)
                functionStringSingle = "DecodeFunctions.intArrayToIntArray(groupOfDTC, 'uint8', 'int8')"

                encodeFunction = "encoded = {1}".format(longName,functionStringSingle)

        funcString = requestFuncTemplate.format(shortName,
                                                serviceId,
												encodeFunction)
        exec(funcString)
        return locals()[shortName]

    ##
    # @brief method to create the function to check the positive response for validity
    @staticmethod
    def create_checkPositiveResponseFunction(diagServiceElement, xmlElements):
        responseId = 0
        diagnosticId = 0

        responseIdStart = 0
        responseIdEnd = 0

        shortName = diagServiceElement.find('SHORT-NAME').text
        checkFunctionName = "check_{0}".format(shortName)
        positiveResponseElement = xmlElements[(diagServiceElement.find('POS-RESPONSE-REFS')).find('POS-RESPONSE-REF').attrib['ID-REF']]

        paramsElement = positiveResponseElement.find('PARAMS')

        totalLength = 0

        for param in paramsElement:
            try:
                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except AttributeError:
                    pass

                startByte = int(param.find('BYTE-POSITION').text)

                if(semantic == 'SERVICE-ID'):
                    responseId = int(param.find('CODED-VALUE').text)
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    responseIdStart = startByte
                    responseIdEnd = startByte + listLength
                    totalLength += listLength
                else:
                    pass
            except:
                #print(sys.exc_info())
                pass

        checkFunctionString = checkFunctionTemplate.format(checkFunctionName, # 0
                                                           responseId, # 1
                                                           responseIdStart, # 2
                                                           responseIdEnd, # 3
                                                           totalLength) # 4
        exec(checkFunctionString)
        return locals()[checkFunctionName]

    ##
    # @brief method to encode the positive response from the raw type to it physical representation
    @staticmethod
    def create_encodePositiveResponseFunction(diagServiceElement, xmlElements):
        # There's nothing to extract here! The only value in the response is the DID, checking of which is handled in the check function, 
        # so must be present and ok. This function is only required to return the default None response.
		
        shortName = diagServiceElement.find('SHORT-NAME').text
        encodePositiveResponseFunctionName = "encode_{0}".format(shortName)
		
        encodeFunctionString = encodePositiveResponseFuncTemplate.format(encodePositiveResponseFunctionName) # 0
        exec(encodeFunctionString)
        return locals()[encodePositiveResponseFunctionName]

    ##
    # @brief method to create the negative response function for the service element
    @staticmethod
    def create_checkNegativeResponseFunction(diagServiceElement, xmlElements):
        shortName = diagServiceElement.find('SHORT-NAME').text
        check_negativeResponseFunctionName = "check_negResponse_{0}".format(shortName)

        negativeResponsesElement = diagServiceElement.find('NEG-RESPONSE-REFS')

        negativeResponseChecks = []

        for negativeResponse in negativeResponsesElement:
            negativeResponseRef = xmlElements[negativeResponse.attrib['ID-REF']]

            negativeResponseParams = negativeResponseRef.find('PARAMS')

            for param in negativeResponseParams:

                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except:
                    semantic = None

                if semantic == 'SERVICE-ID':
                    serviceId = param.find('CODED-VALUE').text
                    start = int(param.find('BYTE-POSITION').text)
                    diagCodedType = param.find('DIAG-CODED-TYPE')
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength/8)
                    end = start + listLength

                    checkString = "if input[{0}:{1}] == [{2}]: raise Exception(\"Detected negative response: {{0}}\".format(str([hex(n) for n in input])))".format(start,
                                                                                                                                                                   end,
                                                                                                                                                                   serviceId)
                    negativeResponseChecks.append(checkString)

                    pass
                pass

        negativeResponseFunctionString = negativeResponseFuncTemplate.format(check_negativeResponseFunctionName,
                                                                             "\n....".join(negativeResponseChecks))
        exec(negativeResponseFunctionString)
        return locals()[check_negativeResponseFunctionName]



#!/usr/bin/env python
from uds.Uds import Config
from can_connect.CanTp import CanTp
from os import path
import pdb


##
# @brief class for creating Tp objects
class TpFactory(object):

    configType = ''
    configParameters = []

    config = None

    ##
    # @brief method to create the different connection types
    @staticmethod
    def __call__(tpType, configPath=None, **kwargs):

        #TpFactory.loadConfiguration(configPath)

        if(tpType == "CAN"):
            return CanTp(configPath=configPath, **kwargs)
        elif(tpType == "DoIP"):
            raise NotImplementedError("DoIP transport not currently supported")
        elif(tpType == "K-LINE"):
            raise NotImplementedError("K-Line Transport not currently supported")
        # elif(tpType == "LIN"):
        #     return LinTp(configPath=configPath, **kwargs)
        elif(tpType == "FLEXRAY"):
            raise NotImplementedError("FlexRay Transport not currently supported")
        # elif(tpType == "TEST"):
        #     return TestTp()
        else:
            raise Exception("Unknown transport type selected")

    @staticmethod
    def loadConfiguration(configPath=None):

        #load the base config
        baseConfig = path.dirname(__file__) + "\config.ini"
        config = Config()
        if path.exists(baseConfig):
            config.read(baseConfig)
        else:
            raise FileNotFoundError("No base config file")

        # check the config path
        if configPath is not None:
            if path.exists(configPath):
                config.read(configPath)
            else:
                raise FileNotFoundError("specified config not found")

        TpFactory.config = config


if __name__ == "__main__":

    pass



#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from uds.uds_config_tool.SupportedServices.iContainer import iContainer
from types import MethodType


class ECUResetContainer(object):

    __metaclass__ = iContainer

    def __init__(self):
        self.requestFunctions = {}
        self.checkFunctions = {}
        self.negativeResponseFunctions = {}
        self.positiveResponseFunctions = {}

    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.ecuReset("something","something else") It does not operate
    # on this instance of the container class.
    @staticmethod
    def __ecuReset(target, parameter, suppressResponse=False, **kwargs):

        # Note: ecuReset does not show support for multiple DIDs in the spec, so this is handling only a single DID with data record.
        requestFunction = target.ecuResetContainer.requestFunctions[parameter]
        if parameter in target.ecuResetContainer.checkFunctions:
            checkFunction = target.ecuResetContainer.checkFunctions[parameter]
        else:
            checkFunction = None
        negativeResponseFunction = target.ecuResetContainer.negativeResponseFunctions[parameter]
        if parameter in target.ecuResetContainer.positiveResponseFunctions:
            positiveResponseFunction = target.ecuResetContainer.positiveResponseFunctions[parameter]
        else:
            positiveResponseFunction = None

        # Call the sequence of functions to execute the ECU Reset request/response action ...
        # ==============================================================================

        if checkFunction is None or positiveResponseFunction is None:
            suppressResponse = True

        # Create the request. Note: we do not have to pre-check the dataRecord as this action is performed by 
        # the recipient (the response codes 0x?? and 0x?? provide the necessary cover of errors in the request) ...
        request = requestFunction(suppressResponse)

        if suppressResponse == False:
            # Send request and receive the response ...
            response = target.send(request, responseRequired=True) # ... this returns a single response
            negativeResponseFunction(response)  # ... throws an exception to be handled at a higher level if a negative response is received

            # We have a positive response so check that it makes sense to us ...
            checkFunction(response)

            # All is still good, so return the response (currently this function does nothing, but including it here as a hook in case that changes) ...
            return positiveResponseFunction(response)
			
		# ... else ...
        # Send request and receive the response ...
        response = target.send(request,responseRequired=False) # ... this suppresses any response handling (not expected)
        return

    def bind_function(self, bindObject):
        bindObject.ecuReset = MethodType(self.__ecuReset, bindObject)

    def add_requestFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.requestFunctions[dictionaryEntry] = aFunction

    def add_checkFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.checkFunctions[dictionaryEntry] = aFunction

    def add_negativeResponseFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.negativeResponseFunctions[dictionaryEntry] = aFunction

    def add_positiveResponseFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.positiveResponseFunctions[dictionaryEntry] = aFunction


#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from abc import ABCMeta, abstractmethod

class iContainer(ABCMeta):

    @abstractmethod
    def add_requestFunction(self, aFunction, dictionaryEntry):
        raise NotImplementedError("add_requestFucntion not implemented")

    @abstractmethod
    def add_checkFunction(self, aFunction, dictionaryEntry):
        raise NotImplementedError("add_checkFunction not implemented")

    @abstractmethod
    def add_negativeResponseFunction(self, aFunction, dictionaryEntry):
        raise NotImplementedError("add_negativeResponseFunction not implemented")

    @abstractmethod
    def add_positiveResponseFunction(self, aFunction, dictionaryEntry):
        raise NotImplementedError("add_positiveResponseFunction not implemented")



from uds.Uds import Uds
import time

# Constants
MAX_RETRIES = 10  # Number of retries
RETRY_DELAY = 0.5  # Delay between retries in seconds

def send_with_retry(uds_instance, request):
    for attempt in range(MAX_RETRIES):
        try:
            response = uds_instance.send(request)
            return response  # Return the successful response
        except Exception as e:
            if "Timeout in waiting for message" in str(e):
                print(f"Attempt {attempt + 1} failed: {e}. Retrying...")
                time.sleep(RETRY_DELAY)  # Wait before retrying
            else:
                raise  # Re-raise other exceptions immediately
    raise Exception("Max retries reached. Communication failed.")

def initialize_uds(driver_type, request_id, response_id, **kwargs):
    """
    Initialize the UDS instance based on driver type and user inputs.

    Parameters:
    - driver_type (str): The type of driver to use ('kvaser', 'peak', or 'vector').
    - request_id (int): The request ID.
    - response_id (int): The response ID.
    - **kwargs: Additional parameters based on the driver type.

    Returns:
    - Uds instance
    """
    if driver_type == "peak":
        return Uds(reqId=request_id, resId=response_id, interface="peak", device=kwargs.get("device", "PCAN_USBBUS1"))
    elif driver_type == "kvaser":
        return Uds(reqId=request_id, resId=response_id, transportProtocol="CAN", interface="kvaser", channel=kwargs.get("channel", 0), bitrate=kwargs.get("bitrate", 500000))
    elif driver_type == "vector":
        return Uds(reqId=request_id, resId=response_id, transportProtocol="CAN", interface="vector", channel=kwargs.get("channel", 0), app_name=kwargs.get("app_name", "BALCAN"))
    else:
        raise ValueError("Unsupported driver type. Please select 'kvaser', 'peak', or 'vector'.")






#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


defaultTPTimeout = 10  # ... (s): default to sending tester present 10s after last request (if required) - note: this includes after previous TesterPresent for repeating operation

from uds.uds_config_tool.SupportedServices.iContainer import iContainer
from types import MethodType
import time


class DiagnosticSessionControlContainer(object):
    __metaclass__ = iContainer

    def __init__(self):
        self.requestFunctions = {}
        self.checkFunctions = {}
        self.negativeResponseFunctions = {}
        self.positiveResponseFunctions = {}

        self.testerPresent = {}
        self.currentSession = None
        self.lastSend = None
		
    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.diagnosticSessionControl("session type") It does not operate
    # on this instance of the container class.
    @staticmethod
    def __diagnosticSessionControl(target, parameter, suppressResponse=False, testerPresent=False, tpTimeout=defaultTPTimeout, **kwargs):

        # Note: diagnosticSessionControl does not show support for multiple DIDs in the spec, so this is handling only a single DID with data record.
        requestFunction = target.diagnosticSessionControlContainer.requestFunctions[parameter]
        if parameter in target.diagnosticSessionControlContainer.checkFunctions:
            checkFunction = target.diagnosticSessionControlContainer.checkFunctions[parameter]
        else:
            checkFunction = None
        negativeResponseFunction = target.diagnosticSessionControlContainer.negativeResponseFunctions[parameter]
        positiveResponseFunction = target.diagnosticSessionControlContainer.positiveResponseFunctions[parameter]
        if parameter in target.diagnosticSessionControlContainer.positiveResponseFunctions:
            positiveResponseFunction = target.diagnosticSessionControlContainer.positiveResponseFunctions[parameter]
        else:
            positiveResponseFunction = None

        # Call the sequence of functions to execute the Diagnostic Session Control request/response action ...
        # ==============================================================================
		
        # Code additions to support interaction with tester present for a given diagnostic session ...
        target.diagnosticSessionControlContainer.currentSession = parameter
		# Note: if testerPresent is set, then timeout is checked every second, so timeout values less than second will always be equivalent to one second.
        target.diagnosticSessionControlContainer.testerPresent[parameter] = {'reqd': True,'timeout':tpTimeout} if testerPresent else {'reqd': False,'timeout':None}
        # Note: lastSend is initialised via a call to __sessionSetLastSend() when send is called
        if testerPresent:
            target.testerPresentThread()

        if checkFunction is None or positiveResponseFunction is None:  # ... i.e. we only have a send_only service specified in the ODX
            suppressResponse = True

        # Create the request. Note: we do not have to pre-check the dataRecord as this action is performed by 
        # the recipient (the response codes 0x?? and 0x?? provide the necessary cover of errors in the request) ...
        request = requestFunction(suppressResponse)

        if suppressResponse == False:        # Send request and receive the response ...
            response = target.send(request,responseRequired=True) # ... this returns a single response
            negativeResponseFunction(response)  # ... throws an exception to be handled at a higher level if a negative response is received

            # We have a positive response so check that it makes sense to us ...
            checkFunction(response)

            # All is still good, so return the response (currently this function does nothing, but including it here as a hook in case that changes) ...
            return positiveResponseFunction(response)
			
		# ... else ...
        # Send request and receive the response ...
        response = target.send(request,responseRequired=False) # ... this suppresses any response handling (not expected)
        return

    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.testerPresentSessionRecord() It does not operate
    # on this instance of the container class.
    # The purpose of this method is to inform the caller of requirement for tester present messages, and if required, at what frequency (in ms)
    @staticmethod
    def __testerPresentSessionRecord(target, **kwargs):
        sessionType = target.diagnosticSessionControlContainer.currentSession
        if sessionType is None:
            sessionType = 'Default Session'
        return target.diagnosticSessionControlContainer.testerPresent[sessionType] if sessionType in target.diagnosticSessionControlContainer.testerPresent else {'reqd': False,'timeout':None}

    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.testerPresentSessionRecord() It does not operate
    # on this instance of the container class.
    # The purpose of this method is to record the last send time (any message) for the current diagnostic session.
    @staticmethod
    def __sessionSetLastSend(target, **kwargs):
        target.diagnosticSessionControlContainer.lastSend = int(round(time.time()))  # ... in seconds

    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.testerPresentSessionRecord() It does not operate
    # on this instance of the container class.
    # The purpose of this method is to record the last send time (any message) for the current diagnostic session.
    @staticmethod
    def __testerPresentDisable(target, **kwargs):
        sessionType = target.diagnosticSessionControlContainer.currentSession
        if sessionType is None:
            sessionType = 'Default Session'
        target.diagnosticSessionControlContainer.testerPresent[sessionType] = {'reqd': False,'timeout':None}
        target.diagnosticSessionControlContainer.lastSend = None

    ##
    # @brief this method is bound to an external Uds object, referenced by target, so that it can be called
    # as one of the in-built methods. uds.testerPresentSessionRecord() It does not operate
    # on this instance of the container class.
    # The purpose of this method is to inform the caller of the time (in seconds) since the last message was sent for the current diagnostic session.
    @staticmethod
    def __sessionTimeSinceLastSend(target, **kwargs):
        now = int(round(time.time()))  # ... in seconds
        try:
            return (now - target.diagnosticSessionControlContainer.lastSend)
        except:
            return 0

    def bind_function(self, bindObject):
        bindObject.diagnosticSessionControl = MethodType(self.__diagnosticSessionControl, bindObject)
        # Adding an additional functions to allow internal requests to process testerPresent behaviour required for the diagnostic session ...
        bindObject.testerPresentSessionRecord = MethodType(self.__testerPresentSessionRecord, bindObject)
        bindObject.sessionSetLastSend = MethodType(self.__sessionSetLastSend, bindObject)
        bindObject.testerPresentDisable = MethodType(self.__testerPresentDisable, bindObject)
        bindObject.sessionTimeSinceLastSend = MethodType(self.__sessionTimeSinceLastSend, bindObject)

    def add_requestFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.requestFunctions[dictionaryEntry] = aFunction

    def add_checkFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.checkFunctions[dictionaryEntry] = aFunction

    def add_negativeResponseFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.negativeResponseFunctions[dictionaryEntry] = aFunction

    def add_positiveResponseFunction(self, aFunction, dictionaryEntry):
        if aFunction is not None: # ... allow for a send only version being processed
            self.positiveResponseFunctions[dictionaryEntry] = aFunction




#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


import xml.etree.ElementTree as ET

from uds_tools.UtilityFunctions import isDiagServiceTransmissionOnly
from uds.Uds import Uds
from SupportedServices.DiagnosticSessionControlContainer import DiagnosticSessionControlContainer
from FunctionCreation.DiagnosticSessionControlMethodFactory import DiagnosticSessionControlMethodFactory
from SupportedServices.ECUResetContainer import ECUResetContainer
from FunctionCreation.ECUResetMethodFactory import ECUResetMethodFactory
from SupportedServices.ReadDataByIdentifierContainer import ReadDataByIdentifierContainer
from FunctionCreation.ReadDataByIdentifierMethodFactory import ReadDataByIdentifierMethodFactory
from SupportedServices.WriteDataByIdentifierContainer import WriteDataByIdentifierContainer
from FunctionCreation.WriteDataByIdentifierMethodFactory import WriteDataByIdentifierMethodFactory
from SupportedServices.ClearDTCContainer import ClearDTCContainer
from FunctionCreation.ClearDTCMethodFactory import ClearDTCMethodFactory
from SupportedServices.ReadDTCContainer import ReadDTCContainer
from FunctionCreation.ReadDTCMethodFactory import ReadDTCMethodFactory
from SupportedServices.InputOutputControlContainer import InputOutputControlContainer
from FunctionCreation.InputOutputControlMethodFactory import InputOutputControlMethodFactory
from SupportedServices.RoutineControlContainer import RoutineControlContainer
from FunctionCreation.RoutineControlMethodFactory import RoutineControlMethodFactory
from SupportedServices.RequestDownloadContainer import RequestDownloadContainer
from FunctionCreation.RequestDownloadMethodFactory import RequestDownloadMethodFactory

from SupportedServices.SecurityAccessContainer import SecurityAccessContainer
from FunctionCreation.SecurityAccessMethodFactory import SecurityAccessMethodFactory
from SupportedServices.RequestUploadContainer import RequestUploadContainer
from FunctionCreation.RequestUploadMethodFactory import RequestUploadMethodFactory
from SupportedServices.TransferDataContainer import TransferDataContainer
from FunctionCreation.TransferDataMethodFactory import TransferDataMethodFactory
from SupportedServices.TransferExitContainer import TransferExitContainer
from FunctionCreation.TransferExitMethodFactory import TransferExitMethodFactory
from SupportedServices.TesterPresentContainer import TesterPresentContainer
from FunctionCreation.TesterPresentMethodFactory import TesterPresentMethodFactory
from ISOStandard.ISOStandard import IsoServices, IsoRoutineControlType, IsoInputOutputControlOptionRecord, IsoReadDTCSubfunction, IsoReadDTCStatusMask as Mask

from ISOStandard import IsoServices

def get_serviceIdFromXmlElement(diagServiceElement, xmlElements):

    requestKey = diagServiceElement.find('REQUEST-REF').attrib['ID-REF']
    requestElement = xmlElements[requestKey]
    params = requestElement.find('PARAMS')
    for i in params:
        try:
            if(i.attrib['SEMANTIC'] == 'SERVICE-ID'):
                return int(i.find('CODED-VALUE').text)
        except:
            pass

    return None


def fill_dictionary(xmlElement):
    temp_dictionary = {}
    for i in xmlElement:
        temp_dictionary[i.attrib['ID']] = i

    return temp_dictionary


def createUdsConnection(xmlFile, ecuName, ihexFile=None, **kwargs):

    root = ET.parse(xmlFile)

    # create any supported containers
    diagnosticSessionControlContainer = DiagnosticSessionControlContainer()
    ecuResetContainer = ECUResetContainer()
    rdbiContainer = ReadDataByIdentifierContainer()
    wdbiContainer = WriteDataByIdentifierContainer()
    clearDTCContainer = ClearDTCContainer()
    readDTCContainer = ReadDTCContainer()
    inputOutputControlContainer = InputOutputControlContainer()
    routineControlContainer = RoutineControlContainer()
    requestDownloadContainer = RequestDownloadContainer()
    securityAccessContainer = SecurityAccessContainer()
    requestUploadContainer = RequestUploadContainer()
    transferDataContainer = TransferDataContainer()
    transferExitContainer = TransferExitContainer()
    testerPresentContainer = TesterPresentContainer()
    sessionService_flag = False
    ecuResetService_flag = False
    rdbiService_flag = False
    wdbiService_flag = False
    securityAccess_flag = False
    clearDTCService_flag = False
    readDTCService_flag = False
    ioCtrlService_flag = False
    routineCtrlService_flag = False
    reqDownloadService_flag = False
    reqUploadService_flag = False
    transDataService_flag = False
    transExitService_flag = False
    testerPresentService_flag = False
    xmlElements = {}

    for child in root.iter():
        currTag = child.tag
        try:
            xmlElements[child.attrib['ID']] = child
        except KeyError:
            pass

    for key, value in xmlElements.items():
        if value.tag == 'DIAG-SERVICE':
            serviceId = get_serviceIdFromXmlElement(value, xmlElements)
            sdg = value.find('SDGS').find('SDG')
            humanName = ""
            for sd in sdg:
                try:
                    if sd.attrib['SI'] == 'DiagInstanceName':
                        humanName = sd.text
                except KeyError:
                    pass


            if serviceId == IsoServices.DiagnosticSessionControl:
                sessionService_flag = True

                requestFunc = DiagnosticSessionControlMethodFactory.create_requestFunction(value, xmlElements)
                diagnosticSessionControlContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = DiagnosticSessionControlMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                diagnosticSessionControlContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                checkFunc = DiagnosticSessionControlMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                diagnosticSessionControlContainer.add_checkFunction(checkFunc, humanName)

                positiveResponseFunction = DiagnosticSessionControlMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                diagnosticSessionControlContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.EcuReset:
                ecuResetService_flag = True

                requestFunc = ECUResetMethodFactory.create_requestFunction(value, xmlElements)
                ecuResetContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = ECUResetMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                ecuResetContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                try:
                    transmissionMode = value.attrib['TRANSMISSION-MODE']
                    if transmissionMode == "SEND-ONLY":
                        sendOnly_flag = True
                except:
                    sendOnly_flag = False

                if sendOnly_flag:
                    checkFunc = None
                    positiveResponseFunction = None
                else:
                    checkFunc = ECUResetMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                    positiveResponseFunction = ECUResetMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)

                ecuResetContainer.add_checkFunction(checkFunc, humanName)
                ecuResetContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)
                pass

            elif serviceId == IsoServices.ReadDataByIdentifier:
                rdbiService_flag = True

                # The new code extends the range of functions required, in order to handle RDBI working for concatenated lists of DIDs ...
                requestFunctions = ReadDataByIdentifierMethodFactory.create_requestFunctions(value, xmlElements)
                rdbiContainer.add_requestSIDFunction(requestFunctions[0], humanName)  # ... note: this will now need to handle replication of this one!!!!
                rdbiContainer.add_requestDIDFunction(requestFunctions[1], humanName)


                negativeResponseFunction = ReadDataByIdentifierMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                rdbiContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)


                checkFunctions = ReadDataByIdentifierMethodFactory.create_checkPositiveResponseFunctions(value, xmlElements)
                rdbiContainer.add_checkSIDResponseFunction(checkFunctions[0], humanName)
                rdbiContainer.add_checkSIDLengthFunction(checkFunctions[1], humanName)
                rdbiContainer.add_checkDIDResponseFunction(checkFunctions[2], humanName)
                rdbiContainer.add_checkDIDLengthFunction(checkFunctions[3], humanName)


                positiveResponseFunction = ReadDataByIdentifierMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                rdbiContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.SecurityAccess:
                if isDiagServiceTransmissionOnly(value) == False:
                    requestFunction = SecurityAccessMethodFactory.create_requestFunction(value, xmlElements)
                    securityAccessContainer.add_requestFunction(requestFunction, humanName)

                    negativeResponseFunction = SecurityAccessMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                    securityAccessContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                    checkFunction = SecurityAccessMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                    securityAccessContainer.add_positiveResponseFunction(checkFunction, humanName)

                    securityAccess_flag = True

            elif serviceId == IsoServices.WriteDataByIdentifier:

                wdbiService_flag = True
                requestFunc = WriteDataByIdentifierMethodFactory.create_requestFunction(value, xmlElements)
                wdbiContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = WriteDataByIdentifierMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                wdbiContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                checkFunc = WriteDataByIdentifierMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                wdbiContainer.add_checkFunction(checkFunc, humanName)

                positiveResponseFunction = WriteDataByIdentifierMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                wdbiContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.ClearDiagnosticInformation:
                clearDTCService_flag = True
                requestFunc = ClearDTCMethodFactory.create_requestFunction(value, xmlElements)
                clearDTCContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = ClearDTCMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                clearDTCContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                checkFunc = ClearDTCMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                clearDTCContainer.add_checkFunction(checkFunc, humanName)

                positiveResponseFunction = ClearDTCMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                clearDTCContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.ReadDTCInformation:
                readDTCService_flag = True
                requestFunction, qualifier = ReadDTCMethodFactory.create_requestFunction(value, xmlElements)
                if qualifier != "":
                    readDTCContainer.add_requestFunction(requestFunction, "FaultMemoryRead"+qualifier)

                    negativeResponseFunction = ReadDTCMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                    readDTCContainer.add_negativeResponseFunction(negativeResponseFunction, "FaultMemoryRead"+qualifier)

                    checkFunction = ReadDTCMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                    readDTCContainer.add_checkFunction(checkFunction, "FaultMemoryRead"+qualifier)

                    positiveResponseFunction = ReadDTCMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                    readDTCContainer.add_positiveResponseFunction(positiveResponseFunction, "FaultMemoryRead"+qualifier)

            elif serviceId == IsoServices.InputOutputControlByIdentifier:
                ioCtrlService_flag = True
                requestFunc, qualifier  = InputOutputControlMethodFactory.create_requestFunction(value, xmlElements)
                if qualifier != "":
                    inputOutputControlContainer.add_requestFunction(requestFunc, humanName+qualifier)

                    negativeResponseFunction = InputOutputControlMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                    inputOutputControlContainer.add_negativeResponseFunction(negativeResponseFunction, humanName+qualifier)

                    checkFunc = InputOutputControlMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                    inputOutputControlContainer.add_checkFunction(checkFunc, humanName+qualifier)

                    positiveResponseFunction = InputOutputControlMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                    inputOutputControlContainer.add_positiveResponseFunction(positiveResponseFunction, humanName+qualifier)

            elif serviceId == IsoServices.RoutineControl:
                routineCtrlService_flag = True
                # We need a qualifier, as the human name for the start stop, and results calls are all the same, so they otherwise overwrite each other
                requestFunc, qualifier = RoutineControlMethodFactory.create_requestFunction(value, xmlElements)
                if qualifier != "":
                    routineControlContainer.add_requestFunction(requestFunc, humanName+qualifier)

                    negativeResponseFunction = RoutineControlMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                    routineControlContainer.add_negativeResponseFunction(negativeResponseFunction, humanName+qualifier)

                    checkFunc = RoutineControlMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                    routineControlContainer.add_checkFunction(checkFunc, humanName+qualifier)

                    positiveResponseFunction = RoutineControlMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                    routineControlContainer.add_positiveResponseFunction(positiveResponseFunction, humanName+qualifier)

            elif serviceId == IsoServices.RequestDownload:
                reqDownloadService_flag = True
                requestFunc = RequestDownloadMethodFactory.create_requestFunction(value, xmlElements)
                requestDownloadContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = RequestDownloadMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                requestDownloadContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                checkFunc = RequestDownloadMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                requestDownloadContainer.add_checkFunction(checkFunc, humanName)

                positiveResponseFunction = RequestDownloadMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                requestDownloadContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.RequestUpload:
                reqUploadService_flag = True
                requestFunc = RequestUploadMethodFactory.create_requestFunction(value, xmlElements)
                requestUploadContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = RequestUploadMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                requestUploadContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                checkFunc = RequestUploadMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                requestUploadContainer.add_checkFunction(checkFunc, humanName)

                positiveResponseFunction = RequestUploadMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                requestUploadContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.TransferData:
                transDataService_flag = True
                requestFunc = TransferDataMethodFactory.create_requestFunction(value, xmlElements)
                transferDataContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = TransferDataMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                transferDataContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                checkFunc = TransferDataMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                transferDataContainer.add_checkFunction(checkFunc, humanName)

                positiveResponseFunction = TransferDataMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                transferDataContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.RequestTransferExit:
                transExitService_flag = True
                requestFunc = TransferExitMethodFactory.create_requestFunction(value, xmlElements)
                transferExitContainer.add_requestFunction(requestFunc, humanName)

                negativeResponseFunction = TransferExitMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                transferExitContainer.add_negativeResponseFunction(negativeResponseFunction, humanName)

                checkFunc = TransferExitMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                transferExitContainer.add_checkFunction(checkFunc, humanName)

                positiveResponseFunction = TransferExitMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                transferExitContainer.add_positiveResponseFunction(positiveResponseFunction, humanName)

            elif serviceId == IsoServices.TesterPresent:
                # Note: Tester Present is presented here as an exposed service, but it will typically not be called directly, as we'll hook it 
                # in to keep the session alive automatically if requested (details to come, but this is just getting the comms into place).
                testerPresentService_flag = True
                requestFunc = TesterPresentMethodFactory.create_requestFunction(value, xmlElements)
                testerPresentContainer.add_requestFunction(requestFunc, "TesterPresent")

                negativeResponseFunction = TesterPresentMethodFactory.create_checkNegativeResponseFunction(value, xmlElements)
                testerPresentContainer.add_negativeResponseFunction(negativeResponseFunction, "TesterPresent")

                checkFunc = TesterPresentMethodFactory.create_checkPositiveResponseFunction(value, xmlElements)
                testerPresentContainer.add_checkFunction(checkFunc, "TesterPresent")

                positiveResponseFunction = TesterPresentMethodFactory.create_encodePositiveResponseFunction(value, xmlElements)
                testerPresentContainer.add_positiveResponseFunction(positiveResponseFunction, "TesterPresent")


    #need to be able to extract the reqId and resId
    outputEcu = Uds(ihexFile=ihexFile, **kwargs)

    # Bind any ECU Reset services that have been found
    if sessionService_flag:
        setattr(outputEcu, 'diagnosticSessionControlContainer', diagnosticSessionControlContainer)
        diagnosticSessionControlContainer.bind_function(outputEcu)

    # Bind any ECU Reset services that have been found
    if ecuResetService_flag:
        setattr(outputEcu, 'ecuResetContainer', ecuResetContainer)
        ecuResetContainer.bind_function(outputEcu)

    # Bind any rdbi services that have been found
    if rdbiService_flag:
        setattr(outputEcu, 'readDataByIdentifierContainer', rdbiContainer)
        rdbiContainer.bind_function(outputEcu)

    # Bind any security access services have been found
    if securityAccess_flag:
        setattr(outputEcu, 'securityAccessContainer', securityAccessContainer)
        securityAccessContainer.bind_function(outputEcu)

    # Bind any wdbi services have been found
    if wdbiService_flag:
        setattr(outputEcu, 'writeDataByIdentifierContainer', wdbiContainer)
        wdbiContainer.bind_function(outputEcu)

    # Bind any clear DTC services that have been found
    if clearDTCService_flag:
        setattr(outputEcu, 'clearDTCContainer', clearDTCContainer)
        clearDTCContainer.bind_function(outputEcu)

    # Bind any read DTC services that have been found
    if readDTCService_flag:
        setattr(outputEcu, 'readDTCContainer', readDTCContainer)
        readDTCContainer.bind_function(outputEcu)

    # Bind any input output control services that have been found
    if ioCtrlService_flag:
        setattr(outputEcu, 'inputOutputControlContainer', inputOutputControlContainer)
        inputOutputControlContainer.bind_function(outputEcu)

    # Bind any routine control services that have been found
    if routineCtrlService_flag:
        setattr(outputEcu, 'routineControlContainer', routineControlContainer)
        routineControlContainer.bind_function(outputEcu)

    # Bind any request download services that have been found
    if reqDownloadService_flag:
        setattr(outputEcu, 'requestDownloadContainer', requestDownloadContainer)
        requestDownloadContainer.bind_function(outputEcu)

    # Bind any request upload services that have been found
    if reqUploadService_flag:
        setattr(outputEcu, 'requestUploadContainer', requestUploadContainer)
        requestUploadContainer.bind_function(outputEcu)

    # Bind any transfer data services that have been found
    if transDataService_flag:
        setattr(outputEcu, 'transferDataContainer', transferDataContainer)
        transferDataContainer.bind_function(outputEcu)

    # Bind any transfer exit data services that have been found
    if transExitService_flag:
        setattr(outputEcu, 'transferExitContainer', transferExitContainer)
        transferExitContainer.bind_function(outputEcu)

    # Bind any tester present services that have been found
    if testerPresentService_flag:
        setattr(outputEcu, 'testerPresentContainer', testerPresentContainer)
        testerPresentContainer.bind_function(outputEcu)

    return outputEcu


if __name__ == "__main__":

    a = createUdsConnection('Bootloader.odx', 'bootloader')

    a.diagnosticSessionControl('Default Session')
    a.ecuReset('Hard Reset',suppressResponse=False)
    a.readDataByIdentifier('ECU Serial Number')
    a.writeDataByIdentifier('ECU Serial Number','ABC0011223344556')
    a.clearDTC([0xF1, 0xC8, 0x55])
    #a.readDTC(IsoReadDTCSubfunction.reportNumberOfDTCByStatusMask, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear)
    #a.readDTC(IsoReadDTCSubfunction.reportDTCByStatusMask, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear)
    #a.readDTC(IsoReadDTCSubfunction.reportDTCSnapshotIdentification, DTCMaskRecord=[0xF1, 0xC8, 0x55], DTCSnapshotRecordNumber=0x34)
    #a.readDTC(IsoReadDTCSubfunction.reportDTCSnapshotRecordByDTCNumber, DTCMaskRecord=[0xF1, 0xC8, 0x55], DTCSnapshotRecordNumber=0x34)
    #a.readDTC(IsoReadDTCSubfunction.reportDTCSnapshotRecordByRecordNumber, DTCSnapshotRecordNumber=0x34)
    #a.readDTC(IsoReadDTCSubfunction.reportDTCExtendedDataRecordByDTCNumber, DTCMaskRecord=[0xF1, 0xC8, 0x55], DTCExtendedRecordNumber=0x12)
    #a.readDTC(IsoReadDTCSubfunction.reportNumberOfDTCBySeverityMaskRecord, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear, DTCSeverityMaskRecord=Mask.confirmedDtc)
    #a.readDTC(IsoReadDTCSubfunction.reportDTCBySeverityMaskRecord, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear, DTCSeverityMaskRecord=Mask.confirmedDtc)
    #a.readDTC(IsoReadDTCSubfunction.reportSeverityInformationOfDTC, DTCMaskRecord=[0xF1, 0xC8, 0x55])
    #a.readDTC(IsoReadDTCSubfunction.reportSupportedDTC)
    #a.readDTC(IsoReadDTCSubfunction.reportFirstTestFailedDTC)
    #a.readDTC(IsoReadDTCSubfunction.reportFirstConfirmedDTC)
    #a.readDTC(IsoReadDTCSubfunction.reportMostRecentTestFailedDTC)
    #a.readDTC(IsoReadDTCSubfunction.reportMostRecentConfirmedDTC)
    #a.readDTC(IsoReadDTCSubfunction.reportMirrorMemoryDTCByStatusMask, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear)
    #a.readDTC(IsoReadDTCSubfunction.reportMirrorMemoryDTCExtendedDataRecordByDTCNumber, DTCMaskRecord=[0xF1, 0xC8, 0x55], DTCExtendedRecordNumber=0x12)
    #a.readDTC(IsoReadDTCSubfunction.reportNumberOfMirrorMemoryDTCByStatusMask, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear)
    #a.readDTC(IsoReadDTCSubfunction.reportNumberOfEmissionsRelatedOBDDTCByStatusMask, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear)
    #a.readDTC(IsoReadDTCSubfunction.reportEmissionsRelatedOBDDTCByStatusMask, DTCStatusMask=Mask.confirmedDtc + Mask.testFailedSinceLastClear)
    a.inputOutputControl('Booster Target Speed',IsoInputOutputControlOptionRecord.adjust,[8000])
    a.routineControl('Erase Memory',IsoRoutineControlType.startRoutine,[('memoryAddress',[0x01]),('memorySize',[0xF000])])
    a.requestDownload(FormatIdentifier=[0x00],MemoryAddress=[0x40, 0x03, 0xE0, 0x00],MemorySize=[0x00, 0x00, 0x0E, 0x56])
    #a.requestUpload(FormatIdentifier=[0x00],MemoryAddress=[0x40, 0x03, 0xE0, 0x00],MemorySize=[0x00, 0x00, 0x0E, 0x56])   # Not tested or runnable at present
    a.transferData(0x01,[0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF])
    a.transferExit([0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF])
    a.testerPresent(suppressResponse=False)



from abc import ABCMeta, abstractmethod


class iTp(object):
    __metaclass__ = ABCMeta

    ##
    # @brief interface method
    @abstractmethod
    def send(self, payload):
        raise NotImplementedError("send function not implemented")

    ##
    # @brief interface method
    @abstractmethod
    def recv(self, timeout_ms):
        raise NotImplementedError("receive function not implemented")

    ##
    # @brief interface method
    @abstractmethod
    def closeConnection(self):
        raise NotImplementedError("closeConnection function not implemented")


# import DecodeFunctions 

from uds_tools.DecodeFunctions import intArrayToIntArray
from uds_tools.ISOStandard import IsoDataFormatIdentifier
from struct import pack, unpack
from time import sleep, time
from enum import IntEnum
import hashlib
import os


class ihexRecordType(IntEnum):

    Data = 0x00
    EndOfFile = 0x01
    ExtendedSegmentAddress = 0x02
    StartSegmentAddress = 0x03
    ExtendedLinearAddress = 0x04
    StartLinearAddress = 0x05


class ihexData(object):

    def __init__(self):

        self.__startAddress = 0
        self.__size = 0
        self.__data = []
        self.__sendChunksize = None
        self.__sendChunks = None

    @property
    def startAddress(self):
        return self.__startAddress

    @startAddress.setter
    def startAddress(self, value):
        self.__startAddress = value

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, value):
        self.__data = value

    @property
    def dataLength(self):
        return len(self.__data)

    @property
    def transmitChunksize(self):
        return self.__sendChunksize

    @startAddress.setter
    def transmitChunksize(self, value):
        # TODO: need to check permitted ranges if any!!!
        if self.__sendChunksize != value:
            self.__sendChunks = None  # ... send chunks needs re-calculating if required, so force this by setting to null here.
        self.__sendChunksize = value

    def transmitChunks(self, sendChunksize=None):  # ... initialising or re-setting of the chunk size is allowed here for convenience.
        if sendChunksize is not None:
            self.transmitChunksize = sendChunksize
        if self.__sendChunksize is not None and self.__data != []:
            self.__sendChunks = []
            chunk = []
            count = 0
            for i in range(0, len(self.__data)):
                chunk.append(self.__data[i])
                count += 1
                if count == self.__sendChunksize:
                    self.__sendChunks.append(chunk)
                    chunk = []
                    count = 0
            if len(chunk) != 0:
                self.__sendChunks.append(chunk)
        if self.__sendChunks is None:
            return []		
        return self.__sendChunks

    @property
    def transmitLength(self):  # ... this is dataLength encoded
        return intArrayToIntArray([self.dataLength], 'int32', 'int8')  # ... length calc'd as [0x00, 0x01, 0x4F, 0xe4] as expected

    def addData(self, value):
        self.__data += value
        self.__sendChunks = None

    def getDataFromAddress(self, address, size):
        raise NotImplemented("getDataFromAddress Not yet implemented")

    @property
    def transmitAddress(self):
        return intArrayToIntArray([self.__startAddress], 'int32', 'int8')


class ihexFile(object):

    def __init__(self, filename=None, padding=0xFF, continuousBlocking=True):

        hexFile = open(filename, 'r')

        self.__blocks = []

        eof_flag = False
        linecount = 1

        nextAddress = None

        currentBlock = None
        baseAddress = 0

        self.__sendChunksize = None

        while not eof_flag:
            line = hexFile.readline()
            linecount += 1

            if line[0] != ":":
                hexFile.close()
                raise Exception("Unexpected line on line {0}".format(linecount))

            lineArray = bytes.fromhex(line[1:])

            # get the main data
            index = 0
            dataLength = lineArray[index]
            index += 1
            address = (lineArray[index] << 8) | (lineArray[index+1])
            index += 2
            recordType = lineArray[index]
            index += 1
            data = lineArray[index: index+dataLength]
            index += dataLength
            checksum = lineArray[index]

            calculatedChecksum = 0

            for i in range(len(lineArray)-1):
                calculatedChecksum = (calculatedChecksum + lineArray[i]) % 256

            calculatedChecksum = ( ~calculatedChecksum + 1 ) %256

            if calculatedChecksum != checksum:
                hexFile.close()
                raise Exception("Checksum on line {0} does not match. Actual: {1}, Calculated: {2}".format(linecount,
                                                                                                           checksum,
                                                                                                           calculatedChecksum))

            # print("Length: {0:#x}, Address: {1:#x}, recordType: {2:#x}, data: {3}, checksum: {4:#x}, calculatedChecksum: {5:#x}".format(dataLength,
            #                                                                                                                             address,
            #                                                                                                                             recordType,
            #                                                                                                                             data,
            #                                                                                                                             checksum,
            #                                                                                                                             calculatedChecksum))

            if recordType == ihexRecordType.Data:   # ... We match data first as it's the most common record type, so more efficient
                if nextAddress is None:
                    currentBlock.startAddress = baseAddress + address

                # As each line of data is individually addressed, there may be disconuities present in the data. 
                # If so (i.e. a gap in the addressing), and a continuous record is required, then pad the data.
                # NOTE: by default, padding is expected.
                if nextAddress is not None:
                    if address != nextAddress:
                        if continuousBlocking:
                            paddingBlock = []
                            [paddingBlock.append(padding) for i in range(0, address-nextAddress)]
                            currentBlock.addData(paddingBlock)

                currentBlock.addData(data)
                nextAddress = address + dataLength


            elif recordType == ihexRecordType.ExtendedLinearAddress:   # ... new block - append any existing block to the blocklist or initialise the current block record
                if currentBlock is not None:
                    # IMPORTANT NOTE (possible TODO): Richard indicated that the last data line may need some tail end padding - if that's the case, we would know about it
                    # till here, so the need for such padidng would have to be detected here and added (e.g. check a "required" flag, and if true, run a moduls op on 
                    # block length to detect if padding needed, then add padding bytes, as above in continuousBlocking case).
                    self.__blocks.append(currentBlock)
                currentBlock = ihexData()  #... start the new block
                baseAddress = ((data[0]<< 8) + data[1]) << 16
                nextAddress = None


            elif recordType == ihexRecordType.EndOfFile: # ... add the final block to the block list
                eof_flag = True
                if currentBlock is not None:
                    self.__blocks.append(currentBlock)


            elif recordType == ihexRecordType.ExtendedSegmentAddress:
                hexFile.close()
                raise NotImplemented("Not implemented extended segment address")

            elif recordType == ihexRecordType.StartSegmentAddress:
                hexFile.close()
                raise NotImplemented("Start segment address not implemented")

            elif recordType == ihexRecordType.StartLinearAddress:
                hexFile.close()
                raise NotImplemented("Start linear address not implemented")
        hexFile.close()

    @property
    def dataLength(self):
        return sum([self.__blocks[i].dataLength for i in range(self.numBlocks)])

    @property
    def numBlocks(self):
        return len(self.__blocks)

    @property
    def blocks(self):
        return self.__blocks

    @property
    def transmitChunksize(self):
        return self.__sendChunksize

    @transmitChunksize.setter
    def transmitChunksize(self, value):
        # TODO: need to check permitted ranges if any!!!
        self.__sendChunksize = value
        for block in self.__blocks:
            block.transmitChunksize = value

    def transmitChunks(self, sendChunksize=None):  # ... initialising or re-setting of the chunk size is allowed here for convenience.
        if sendChunksize is not None:
            self.transmitChunksize = sendChunksize		
        return sum([self.__blocks[i].transmitChunks() for i in range(self.numBlocks)],[])

    @property
    def transmitLength(self):  # ... this is dataLength encoded
        return intArrayToIntArray([self.dataLength], 'int32', 'int8')

    @property
    def transmitAddress(self):
        return self.__blocks[0].transmitAddress


if __name__ == "__main__":

    app_blocks = ihexFile("../../test/Uds-Config-Tool/Functional Tests/e400_uds_test_app_e400.hex")
    #print(("found num blocks : ",  app_blocks.numBlocks))
    #print(("len block data[0] : ", app_blocks.block[0].dataLength))
    #print(("len block data[1] : ", app_blocks.block[1].dataLength))

    #smallerChunks = app_blocks.block[0].transmitChunks(sendChunksize=1280)  # ... breaking the data block to transmittable chunks
    #print(("found num small blocks : ", len(smallerChunks)))
    #smallerChunks = app_blocks.block[1].transmitChunks(sendChunksize=1280)  # ... breaking the data block to transmittable chunks
    #print(("found num small blocks : ", len(smallerChunks)))

    #transmitChunks = sum([app_blocks.block[i].transmitChunks(sendChunksize=1280) for i in range(app_blocks.numBlocks)],[])
    #print(("transmit total chunks : ", len(transmitChunks)))
	
    #print(("transmit start address (all) : ", app_blocks.transmitAddress))
    #print(("transmit start address (block 0) : ", app_blocks.block[0].transmitAddress))
    #print(("transmit start address (block 1) : ", app_blocks.block[1].transmitAddress))

    #transmitLength = sum([len(app_blocks.block[i].data) for i in range(app_blocks.numBlocks)])
    #print(("data length (total) : ",      app_blocks.dataLength))
    #print(("transmit length (total) : ",  app_blocks.transmitLength))
    #print(("transmit length (block 0): ", app_blocks.block[0].transmitLength))

    """ Examples - see also unittest_TransDataFunctions.py for ihex test cases ...
    app_blocks = ihexFile("../../test/Uds-Config-Tool/Functional Tests/TGT-ASSY-1383_v2.1.0_sbl.hex")
    e400 = createUdsConnection("Bootloader.odx", "Bootloader", reqId=0x600, resId=0x650, interface="peak")
    a = e400.requestDownload([IsoDataFormatIdentifier.noCompressionMethod], app_blocks.transmitAddress, app_blocks.transmitLength)
	....
	app_blocks.transmitChunksize = 1280
	a = e400.transferData(transferBlock=app_blocks.block[0])
	# ... or ...
	a = e400.transferData(transferBlocks=app_blocks)
    ....
    a = e400.transferExit()
    """
	
	





#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from uds.uds_config_tool import DecodeFunctions
import sys
from uds.uds_config_tool.FunctionCreation.iServiceMethodFactory import IServiceMethodFactory


# When encode the dataRecord for transmission we allow for multiple elements in the data record,
# i.e. IO Ctrl always has the option and mask records in the request
requestFuncTemplate = str("def {0}(dataRecord):\n"
                          "    encoded = []\n"
                          "    if type(dataRecord) == list and type(dataRecord[0]) == tuple:\n"
                          "        drDict = dict(dataRecord)\n"
                          "        {4}\n"
                          "{5}\n"
                          "    return {1} + {2} + {3} + encoded")											 

checkFunctionTemplate = str("def {0}(input):\n"
                            "    serviceIdExpected = {1}\n"
                            "    diagnosticIdExpected = {2}\n"
                            "    optionRecordExpected = {3}\n"
                            "    serviceId = DecodeFunctions.buildIntFromList(input[{4}:{5}])\n"
                            "    diagnosticId = DecodeFunctions.buildIntFromList(input[{6}:{7}])\n"
                            "    optionRecord = DecodeFunctions.buildIntFromList(input[{8}:{9}])\n"
                            "    if(len(input) != {10}): raise Exception(\"Total length returned not as expected. Expected: {10}; Got {{0}}\".format(len(input)))\n"
                            "    if(serviceId != serviceIdExpected): raise Exception(\"Service Id Received not expected. Expected {{0}}; Got {{1}} \".format(serviceIdExpected, serviceId))\n"
                            "    if(diagnosticId != diagnosticIdExpected): raise Exception(\"Diagnostic Id Received not as expected. Expected: {{0}}; Got {{1}}\".format(diagnosticIdExpected, diagnosticId))\n"
                            "    if(optionRecord != optionRecordExpected): raise Exception(\"Option Record Received not as expected. Expected: {{0}}; Got {{1}}\".format(optionRecordExpected, optionRecord))")

negativeResponseFuncTemplate = str("def {0}(input):\n"
                                   "    {1}")

encodePositiveResponseFuncTemplate = str("def {0}(input):\n"
                                         "    result = {{}}\n"
                                         "    {1}\n"
                                         "    return result")


class InputOutputControlMethodFactory(IServiceMethodFactory):

    ##
    # @brief method to create the request function for the service element
    @staticmethod
    def create_requestFunction(diagServiceElement, xmlElements):
        serviceId = 0
        diagnosticId = 0
        optionRecord = 0

        shortName = "request_{0}".format(diagServiceElement.find('SHORT-NAME').text)
        requestElement = xmlElements[diagServiceElement.find('REQUEST-REF').attrib['ID-REF']]
        paramsElement = requestElement.find('PARAMS')

        encodeFunctions = []
        encodeFunction = ""

        for param in paramsElement:
            semantic = None
            try:
                semantic = param.attrib['SEMANTIC']
            except AttributeError:
                pass

            if(semantic == 'SERVICE-ID'):
                serviceId = [int(param.find('CODED-VALUE').text)]
            elif(semantic == 'ID'):
                diagnosticId = DecodeFunctions.intArrayToIntArray([int(param.find('CODED-VALUE').text)], 'int16', 'int8')
            elif(semantic == 'SUBFUNCTION'):
                optionRecord = DecodeFunctions.intArrayToIntArray([int(param.find('CODED-VALUE').text)], 'int8', 'int8')
            elif semantic == 'DATA':
                dataObjectElement = xmlElements[(param.find('DOP-REF')).attrib['ID-REF']]
                longName = param.find('LONG-NAME').text
                bytePosition = int(param.find('BYTE-POSITION').text)
                # Catching any exceptions where we don't know the type - these will fail elsewhere, but at least we can test what does work.
                try:
                    encodingType = dataObjectElement.find('DIAG-CODED-TYPE').attrib['BASE-DATA-TYPE']
                except:
                    encodingType = "unknown"  # ... for now just drop into the "else" catch-all
                if(encodingType) == "A_ASCIISTRING":
                    functionStringList = "DecodeFunctions.stringToIntList(drDict['{0}'], None)".format(longName)
                    functionStringSingle = "DecodeFunctions.stringToIntList(dataRecord, None)"
                elif(encodingType) == "A_INT8":
                    functionStringList = "DecodeFunctions.intArrayToIntArray(drDict['{0}'], 'int8', 'int8')".format(longName)
                    functionStringSingle = "DecodeFunctions.intArrayToIntArray(dataRecord, 'int8', 'int8')"
                elif(encodingType) == "A_INT16":
                    functionStringList = "DecodeFunctions.intArrayToIntArray(drDict['{0}'], 'int16', 'int8')".format(longName)
                    functionStringSingle = "DecodeFunctions.intArrayToIntArray(dataRecord, 'int16', 'int8')"
                elif(encodingType) == "A_INT32":
                    functionStringList = "DecodeFunctions.intArrayToIntArray(drDict['{0}'], 'int32', 'int8')".format(longName)
                    functionStringSingle = "DecodeFunctions.intArrayToIntArray(dataRecord, 'int32', 'int8')"
                elif(encodingType) == "A_UINT8":
                    functionStringList = "DecodeFunctions.intArrayToIntArray(drDict['{0}'], 'uint8', 'int8')".format(longName)
                    functionStringSingle = "DecodeFunctions.intArrayToIntArray(dataRecord, 'uint8', 'int8')"
                elif(encodingType) == "A_UINT16":
                    functionStringList = "DecodeFunctions.intArrayToIntArray(drDict['{0}'], 'uint16', 'int8')".format(longName)
                    functionStringSingle = "DecodeFunctions.intArrayToIntArray(dataRecord, 'uint16', 'int8')"
                elif(encodingType) == "A_UINT32":
                    functionStringList = "DecodeFunctions.intArrayToIntArray(drDict['{0}'], 'uint32', 'int8')".format(longName)
                    functionStringSingle = "DecodeFunctions.intArrayToIntArray(dataRecord, 'uint32', 'int8')"
                else:
                    functionStringList = "drDict['{0}']".format(longName)
                    functionStringSingle = "dataRecord"

                # 
                encodeFunctions.append("encoded += {1}".format(longName,
                                                                 functionStringList))
                encodeFunction = "    else:\n        encoded = {1}".format(longName,functionStringSingle)

        funcString = requestFuncTemplate.format(shortName,
                                                serviceId,
                                                diagnosticId,
                                                optionRecord,
												"\n        ".join(encodeFunctions),  # ... handles input via list
												encodeFunction)                  # ... handles input via single value

        exec(funcString)
        return (locals()[shortName],str(optionRecord))



    ##
    # @brief method to create the function to check the positive response for validity
    @staticmethod
    def create_checkPositiveResponseFunction(diagServiceElement, xmlElements):
        # Some services are present in the ODX in both response and send only versions (with the same short name, so one will overwrite the other).
        # Avoiding the overwrite by ignoring the send-only versions, i.e. these are identical other than postivie response details being missing.
        try:
            if diagServiceElement.attrib['TRANSMISSION-MODE'] == 'SEND-ONLY':
                return None
        except:
            pass

        responseId = 0
        diagnosticId = 0
        optionRecord = 0

        responseIdStart = 0
        responseIdEnd = 0
        diagnosticIdStart = 0
        diagnosticIdEnd = 0
        optionRecordStart = 0
        optionRecordEnd = 0

        shortName = diagServiceElement.find('SHORT-NAME').text
        checkFunctionName = "check_{0}".format(shortName)
        positiveResponseElement = xmlElements[(diagServiceElement.find('POS-RESPONSE-REFS')).find('POS-RESPONSE-REF').attrib['ID-REF']]

        paramsElement = positiveResponseElement.find('PARAMS')

        totalLength = 0
        powerDownTimeLen = 0

        for param in paramsElement:
            try:
                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except AttributeError:
                    pass

                startByte = int(param.find('BYTE-POSITION').text)

                if(semantic == 'SERVICE-ID'):
                    responseId = int(param.find('CODED-VALUE').text)
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    responseIdStart = startByte
                    responseIdEnd = startByte + listLength
                    totalLength += listLength
                elif(semantic == 'ID'):
                    diagnosticId = int(param.find('CODED-VALUE').text)
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    diagnosticIdStart = startByte
                    diagnosticIdEnd = startByte + listLength
                    totalLength += listLength
                elif(semantic == 'SUBFUNCTION'):
                    optionRecord = int(param.find('CODED-VALUE').text)
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    optionRecordStart = startByte
                    optionRecordEnd = startByte + listLength
                    totalLength += listLength
                elif(semantic == 'DATA'):
                    dataObjectElement = xmlElements[(param.find('DOP-REF')).attrib['ID-REF']]
                    if(dataObjectElement.tag == "DATA-OBJECT-PROP"):
                        start = int(param.find('BYTE-POSITION').text)
                        bitLength = int(dataObjectElement.find('DIAG-CODED-TYPE').find('BIT-LENGTH').text)
                        listLength = int(bitLength/8)
                        totalLength += listLength
                    else:
                        pass
                else:
                    pass
            except:
                #print(sys.exc_info())
                pass


        checkFunctionString = checkFunctionTemplate.format(checkFunctionName, # 0
                                                           responseId, # 1
                                                           diagnosticId, # 2
                                                           optionRecord, # 3
                                                           responseIdStart, # 4
                                                           responseIdEnd, # 5
                                                           diagnosticIdStart, # 6
                                                           diagnosticIdEnd, # 7
                                                           optionRecordStart, # 8
                                                           optionRecordEnd, # 9
                                                           totalLength) # 10
        exec(checkFunctionString)
        return locals()[checkFunctionName]


    ##
    # @brief method to encode the positive response from the raw type to it physical representation
    @staticmethod
    def create_encodePositiveResponseFunction(diagServiceElement, xmlElements):
        # Some services are present in the ODX in both response and send only versions (with the same short name, so one will overwrite the other).
        # Avoiding the overwrite by ignoring the send-only versions, i.e. these are identical other than postivie response details being missing.
        try:
            if diagServiceElement.attrib['TRANSMISSION-MODE'] == 'SEND-ONLY':
                return None
        except:
            pass

        # The values in the response include SID and DID, but these do not need to be returned (and already checked in the check function).

        positiveResponseElement = xmlElements[(diagServiceElement.find('POS-RESPONSE-REFS')).find('POS-RESPONSE-REF').attrib['ID-REF']]
		
        shortName = diagServiceElement.find('SHORT-NAME').text
        encodePositiveResponseFunctionName = "encode_{0}".format(shortName)

        params = positiveResponseElement.find('PARAMS')

        encodeFunctions = []

        for param in params:
            try:
                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except AttributeError:
                    pass

                if semantic == 'SUBFUNCTION':
                    longName = param.find('LONG-NAME').text
                    bytePosition = int(param.find('BYTE-POSITION').text)
                    bitLength = int(param.find('DIAG-CODED-TYPE').find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    endPosition = bytePosition + listLength
                    encodingType = param.find('DIAG-CODED-TYPE').attrib['BASE-DATA-TYPE']
                    if(encodingType) == "A_ASCIISTRING":
                        functionString = "DecodeFunctions.intListToString(input[{0}:{1}], None)".format(bytePosition,
                                                                                                        endPosition)
                    else:
                        functionString = "input[{1}:{2}]".format(longName,
                                                                 bytePosition,
                                                                 endPosition)
                    encodeFunctions.append("result['{0}'] = {1}".format(longName,
                                                                        functionString))
                if semantic == 'ID':
                    longName = param.find('LONG-NAME').text
                    bytePosition = int(param.find('BYTE-POSITION').text)
                    bitLength = int(param.find('DIAG-CODED-TYPE').find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    endPosition = bytePosition + listLength
                    encodingType = param.find('DIAG-CODED-TYPE').attrib['BASE-DATA-TYPE']
                    if(encodingType) == "A_ASCIISTRING":
                        functionString = "DecodeFunctions.intListToString(input[{0}:{1}], None)".format(bytePosition,
                                                                                                        endPosition)
                    else:
                        functionString = "input[{1}:{2}]".format(longName,
                                                                 bytePosition,
                                                                 endPosition)
                    encodeFunctions.append("result['{0}'] = {1}".format(longName,
                                                                        functionString))

                if semantic == 'DATA':
                    dataObjectElement = xmlElements[(param.find('DOP-REF')).attrib['ID-REF']]
                    longName = param.find('LONG-NAME').text
                    bytePosition = int(param.find('BYTE-POSITION').text)
                    bitLength = int(dataObjectElement.find('DIAG-CODED-TYPE').find('BIT-LENGTH').text)
                    listLength = int(bitLength / 8)
                    endPosition = bytePosition + listLength
                    encodingType = dataObjectElement.find('DIAG-CODED-TYPE').attrib['BASE-DATA-TYPE']
                    if(encodingType) == "A_ASCIISTRING":
                        functionString = "DecodeFunctions.intListToString(input[{0}:{1}], None)".format(bytePosition,
                                                                                                        endPosition)
                    elif(encodingType == "A_UINT32"):
                        functionString = "input[{1}:{2}]".format(longName,
                                                                 bytePosition,
                                                                 endPosition)
                    else:
                        functionString = "input[{1}:{2}]".format(longName,
                                                                 bytePosition,
                                                                 endPosition)
                    encodeFunctions.append("result['{0}'] = {1}".format(longName,
                                                                        functionString))
            except:
                pass

        encodeFunctionString = encodePositiveResponseFuncTemplate.format(encodePositiveResponseFunctionName,
                                                                         "\n    ".join(encodeFunctions))
        exec(encodeFunctionString)
        return locals()[encodePositiveResponseFunctionName]



    ##
    # @brief method to create the negative response function for the service element
    @staticmethod
    def create_checkNegativeResponseFunction(diagServiceElement, xmlElements):
        # Some services are present in the ODX in both response and send only versions (with the same short name, so one will overwrite the other).
        # Avoiding the overwrite by ignoring the send-only versions, i.e. these are identical other than postivie response details being missing.
        try:
            if diagServiceElement.attrib['TRANSMISSION-MODE'] == 'SEND-ONLY':
                return None
        except:
            pass

        shortName = diagServiceElement.find('SHORT-NAME').text
        check_negativeResponseFunctionName = "check_negResponse_{0}".format(shortName)

        negativeResponsesElement = diagServiceElement.find('NEG-RESPONSE-REFS')

        negativeResponseChecks = []

        for negativeResponse in negativeResponsesElement:
            negativeResponseRef = xmlElements[negativeResponse.attrib['ID-REF']]

            negativeResponseParams = negativeResponseRef.find('PARAMS')

            for param in negativeResponseParams:

                semantic = None
                try:
                    semantic = param.attrib['SEMANTIC']
                except:
                    semantic = None

                if semantic == 'SERVICE-ID':
                    serviceId = param.find('CODED-VALUE').text
                    start = int(param.find('BYTE-POSITION').text)
                    diagCodedType = param.find('DIAG-CODED-TYPE')
                    bitLength = int((param.find('DIAG-CODED-TYPE')).find('BIT-LENGTH').text)
                    listLength = int(bitLength/8)
                    end = start + listLength

                    checkString = "if input[{0}:{1}] == [{2}]: raise Exception(\"Detected negative response: {{0}}\".format(str([hex(n) for n in input])))".format(start,
                                                                                                                                                                   end,
                                                                                                                                                                   serviceId)
                    negativeResponseChecks.append(checkString)

                    pass
                pass

        negativeResponseFunctionString = negativeResponseFuncTemplate.format(check_negativeResponseFunctionName,
                                                                             "\n....".join(negativeResponseChecks))
        exec(negativeResponseFunctionString)
        return locals()[check_negativeResponseFunctionName]



#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from abc import ABCMeta, abstractmethod


##
# @brief this should be static
class IServiceMethodFactory(ABCMeta):

    ##
    # @brief method to create the request function for the service element
    @staticmethod
    @abstractmethod
    def create_requestFunction(diagServiceElement, xmlElements):
        raise NotImplementedError("create_requestFunction not yet implemented")

    ##
    # @brief method to create the function to check the positive response for validity
    @staticmethod
    @abstractmethod
    def create_checkPositiveResponseFunction(diagServiceElement, xmlElements):
        raise NotImplementedError("create_checkPositiveResponseFunction not yet implemented")

    ##
    # @brief method to encode the positive response from the raw type to it physical representation
    @staticmethod
    @abstractmethod
    def create_encodePositiveResponseFunction(diagServiceElement, xmlElements):
        raise NotImplementedError("create_encodePositiveResponseFunction not yet implemented")

    ##
    # @brief method to create the negative response function for the service element
    @staticmethod
    @abstractmethod
    def create_checkNegativeResponseFunction(diagServiceElement, xmlElements):
        raise NotImplementedError("create_checkNegativeResponseFunction not yet implemented")



from setuptools import setup, find_packages

setup(
    name="uds_connect",                    # Name of the package
    version="0.1.5",                       # Package version
    description="A package to interface with UDS via different CAN interfaces",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Omkar Rane",
    author_email="ojrane16124@gmail.com",
    url="https://github.com/yourusername/uds_connect",  # URL of the project's GitHub repo
    license="MIT",                         # License type
    packages=find_packages(),              # Automatically find package directories
    install_requires=[
        "uds",                             # Specify any dependencies here
        "python-can"
    ],
    include_package_data=True,  # Ensures MANIFEST.in files are included
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",               # Python version requirement
)



#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from functools import reduce


def extractBitFromPosition(aInt, position):
    return (aInt & (2**position)) >> position


def extractIntFromPosition(aInt, size, position):
    return (aInt >> position) & ((2**size)-1)


##
# @brief uses the reduce pattern to concatenate the list into a single integer
# tests were performed the assess the benefit of using functional methods
# and the reduce and for loops gave similar times. Using a recursive function
# was almost 10 times slower.
def buildIntFromList(aList):
    return reduce(lambda x, y: (x << 8) + y, aList)


##
# @brief uses list comprehension to deal with the input string
# todo: implement the encoding type
def stringToIntList(aString, encodingType):
    result = []
    [result.append(ord(i)) for i in aString]
    return result


##
# @brief uses the map, reduce pattern to deal with the input list functionally
# todo: implement the encoding type
def intListToString(aList, encodingType):
    return reduce(lambda x, y: x + y, list(map(chr, aList)))


def intArrayToUInt8Array(aArray, inputType):
    return intArrayToIntArray(aArray, inputType, 'int8')


def intArrayToIntArray(aArray, inputType, outputType):
    if (inputType == 'uint32'):
        inputFunc = lambda x: [extractIntFromPosition(x, 8, 24), extractIntFromPosition(x, 8, 16),
                          extractIntFromPosition(x, 8, 8), extractIntFromPosition(x, 8, 0)]
    elif (inputType == 'uint16'):
        inputFunc = lambda x: [extractIntFromPosition(x, 8, 8), extractIntFromPosition(x, 8, 0)]
    elif (inputType == 'uint8'):
        inputFunc = lambda x: [x]
    elif (inputType == 'int32'):
        inputFunc = lambda x: [extractIntFromPosition(x, 8, 24), extractIntFromPosition(x, 8, 16),
                          extractIntFromPosition(x, 8, 8), extractIntFromPosition(x, 8, 0)]
    elif (inputType == 'int16'):
        inputFunc = lambda x: [extractIntFromPosition(x, 8, 8), extractIntFromPosition(x, 8, 0)]
    elif (inputType == 'int8'):
        inputFunc = lambda x: [x]
    else:
        raise TypeError('inputType not currently supported')

    result = reduce(lambda x, y: x + y, list(map(inputFunc, aArray)))

    if(outputType == 'int8'):
        return result
    if(outputType == 'int32'):
        size = 4
        numberOfEntries = int(len(result) / size)
    elif(outputType == 'int16'):
        size = 2
        numberOfEntries = int(len(result) / size)

    output = list(map(buildIntFromList, [result[(i * size):(i * size + size)] for i in range(numberOfEntries)]))
    return output


if __name__ == "__main__":
    a = intArrayToIntArray([0x5AA55AA5, 0xA55AA55A], 'int32', 'int32')
    print(a)
    assert([0x5aa55aa5, 0xa55aa55a] == a)
    a = intArrayToIntArray([0x5AA55AA5, 0xA55AA55A], 'int32', 'int16')
    print(a)
    assert ([0x5aa5, 0x5aa5, 0xa55a, 0xa55a] == a)
    a = intArrayToIntArray([0x5AA55AA5, 0xA55AA55A], 'int32', 'int8')
    print(a)
    assert ([0x5a, 0xa5, 0x5a, 0xa5, 0xa5, 0x5a, 0xa5, 0x5a] == a)
    a = intArrayToIntArray([0x5AA5, 0xA55A], 'int16', 'int16')
    print(a)
    assert ([0x5aa5, 0xA55A] == a)
    a = intArrayToIntArray([0x5AA5, 0xA55A], 'int16', 'int32')
    print(a)
    assert ([0x5aa5A55A] == a)
    a = intArrayToIntArray([0x5AA5, 0xA55A], 'int16', 'int8')
    print(a)
    assert ([0x5a, 0xa5, 0xa5, 0x5a] == a)
    a = intArrayToIntArray([0x5A, 0xA5, 0xA5, 0x5A], 'int8', 'int8')
    print(a)
    assert ([0x5a, 0xa5, 0xa5, 0x5a] == a)
    a = intArrayToIntArray([0x5A, 0xA5, 0xA5, 0x5A], 'int8', 'int16')
    print(a)
    assert ([0x5aa5, 0xa55a] == a)
    a = intArrayToIntArray([0x5A, 0xA5, 0xA5, 0x5A], 'int8', 'int32')
    print(a)
    assert ([0x5aa5a55a] == a)
    a = intArrayToIntArray([0x5A, 0xA5, 0xA5, 0x5A, 0xA5, 0x5A, 0xA5, 0x5A], 'int8', 'int32')
    print(a)
    assert ([0x5aa5a55a, 0xa55aa55a] == a)

    a = intArrayToIntArray([0x01], 'int8', 'int8')
    print(a)

    a = intArrayToUInt8Array([0x01], 'int8')
    print(a)




##
# param: a diag service element
# return: a dictionary with the sdgs data elements
def getSdgsData(diagServiceElement):

    output = {}

    sdgs = diagServiceElement.find("SDGS")
    sdg = sdgs.find("SDG")
    for i in sdg:
        try:
            output[i.attrib['SI']] = i.text
        except:
            pass
    return output

##
# param: a diagServiceElement, an string representing the si attribute
# return: a specific entry from the sdgs params, or none if it does not exist
def getSdgsDataItem(diagServiceElement, itemName):

    outputDict = getSdgsData(diagServiceElement)

    try:
        output = outputDict[itemName]
    except:
        output = None

    return output

##
# param: an xml element
# return: a string with the short name, or None if no short name exists
def getShortName(xmlElement):

    try:
        output = xmlElement.find('SHORT-NAME').text
    except:
        output = None

    return output


##
# param: an xml element
# return: a string with the long name, or None if no long name exists
def getLongName(xmlElement):
    try:
        output = xmlElement.find('LONG-NAME').text
    except:
        output = None

    return output


##
# param: a diag service element, a list of other xmlElements
# return: an integer
def getServiceIdFromDiagService(diagServiceElement, xmlElements):

    requestKey = diagServiceElement.find('REQUEST-REF').attrib['ID-REF']
    requestElement = xmlElements[requestKey]
    params = requestElement.find('PARAMS')
    for i in params:
        try:
            if(i.attrib['SEMANTIC'] == 'SERVICE-ID'):
                return int(i.find('CODED-VALUE').text)
        except:
            pass

    return None


##
# param: a diag service element, a list of other xmlElements
# return: an integer
def getResponseIdFromDiagService(diagServiceElement, xmlElements):

    requestKey = diagServiceElement.find('REQUEST-REF').attrib['ID-REF']
    requestElement = xmlElements[requestKey]
    params = requestElement.find('PARAMS')
    for i in params:
        try:
            if(i.attrib['SEMANTIC'] == 'SERVICE-ID'):
                return int(i.find('CODED-VALUE').text)
        except:
            pass

    return None


##
# params: an xmlElement, the name of a semantic to match
# return: a single parameter matching the semantic, or a list of parameters which match the semantic
def getParamWithSemantic(xmlElement, semanticName):

    output = None

    try:
        params = xmlElement.find("PARAMS")
    except AttributeError:
        return output

    paramsList = []

    for i in params:
        paramSemantic = i.attrib["SEMANTIC"]
        if paramSemantic == semanticName:
            paramsList.append(i)

    if len(paramsList) == 0:
        output = None
    elif len(paramsList) == 1:
        output = paramsList[0]
    else:
        output = paramsList
    return output

##
# params: a diagnostic service element xml entry, and the dictionary of all possible xml elements
# return: if only 1 element, then a single xml element, else a list of xml elements, or none if no positive responses
def getPositiveResponse(diagServiceElement, xmlElements):

    positiveResponseList = []
    try:
        positiveResponseReferences = diagServiceElement.find("POS-RESPONSE-REFS")
    except:
        return None

    if positiveResponseReferences is None:
        return None
    else:
        for i in positiveResponseReferences:
            try:
                positiveResponseList.append(xmlElements[i.attrib["ID-REF"]])
            except:
                pass

    positiveResponseList_length = len(positiveResponseList)
    if positiveResponseList_length == 0:
        return None
    if positiveResponseList_length:
        return positiveResponseList[0]
    else:
        return positiveResponseList


def getDiagObjectProp(paramElement, xmlElements):

    try:
        dopElement = xmlElements[paramElement.find("DOP-REF").attrib["ID-REF"]]
    except:
        dopElement = None

    return dopElement

def getBitLengthFromDop(diagObjectPropElement):

    try:
        bitLength = int(diagObjectPropElement.find("DIAG-CODED-TYPE").find("BIT-LENGTH").text)
    except:
        bitLength = None

    return bitLength

def isDiagServiceTransmissionOnly(diagServiceElement):

    output = getSdgsDataItem(diagServiceElement, "PositiveResponseSuppressed")

    if output is not None:
        if output == "yes":
            return True

    return False


if __name__ == "__main__":

    pass



#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from enum import Enum, IntEnum

class IsoDidNames(Enum):

    bootSoftwareIdentification = "bootSoftwareIdentification"
    applicationSoftwareIdentification = "applicationSoftwareIdentification"

class IsoServices(IntEnum):

    DiagnosticSessionControl = 0x10
    EcuReset = 0x11
    SecurityAccess = 0x27
    CommunicationControl = 0x28
    TesterPresent = 0x3E
    AccessTimingParameter = 0x83
    SecuredDataTransmission = 0x84
    ControlDTCSetting = 0x85
    ResponseOnEvent = 0x86
    LinkControl = 0x87
    ReadDataByIdentifier = 0x22
    ReadMemoryByAddress = 0x23
    ReadScalingDataByIdentifier = 0x24
    ReadDataByPeriodicIdentifier = 0x2A
    DynamicallyDefineDataIdentifier = 0x2C
    WriteDataByIdentifier = 0x2E
    WriteMemoryByAddress = 0x3D
    ClearDiagnosticInformation = 0x14
    ReadDTCInformation = 0x19
    InputOutputControlByIdentifier = 0x2F
    RoutineControl = 0x31
    RequestDownload = 0x34
    RequestUpload = 0x35
    TransferData = 0x36
    RequestTransferExit = 0x37

	
class IsoRoutineControlType(IntEnum):

    startRoutine = 0x01
    stopRoutine = 0x02
    requestRoutineResults = 0x03


class IsoInputOutputControlOptionRecord(IntEnum):

    returnControl = 0x00
    resetToDefault = 0x01
    freezeCurrentState = 0x02
    adjust = 0x03


class IsoReadDTCSubfunction(IntEnum):

    reportNumberOfDTCByStatusMask = 0x01
    reportDTCByStatusMask = 0x02
    reportDTCSnapshotIdentification = 0x03
    reportDTCSnapshotRecordByDTCNumber = 0x04
    reportDTCSnapshotRecordByRecordNumber = 0x05
    reportDTCExtendedDataRecordByDTCNumber = 0x06
    reportNumberOfDTCBySeverityMaskRecord = 0x07
    reportDTCBySeverityMaskRecord = 0x08
    reportSeverityInformationOfDTC = 0x09
    reportSupportedDTC = 0x0A
    reportFirstTestFailedDTC = 0x0B
    reportFirstConfirmedDTC = 0x0C
    reportMostRecentTestFailedDTC = 0x0D
    reportMostRecentConfirmedDTC = 0x0E
    reportMirrorMemoryDTCByStatusMask = 0x0F
    reportMirrorMemoryDTCExtendedDataRecordByDTCNumber = 0x10
    reportNumberOfMirrorMemoryDTCByStatusMask = 0x11
    reportNumberOfEmissionsRelatedOBDDTCByStatusMask = 0x12
    reportEmissionsRelatedOBDDTCByStatusMask = 0x13

	
class IsoReadDTCStatusMask(IntEnum):

    testFailed = 0x01
    testFailedThisMonitoringCycle = 0x02         # ... reserved
    pendingDtc = 0x04                            # ... reserved
    confirmedDtc = 0x08
    testNotCompletedSinceLastClear = 0x10
    testFailedSinceLastClear = 0x20
    testNotCompletedThisMonitoringCycle = 0x40
    warningIndicatorRequested = 0x80             # ... reserved


class IsoDataFormatIdentifier(IntEnum):

    noCompressionMethod = 0x00  # ... for use during request download - all other values are manufacturer specific

	



#!/usr/bin/env python

__author__ = "Richard Clubb"
__copyrights__ = "Copyright 2018, the python-uds project"
__credits__ = ["Richard Clubb"]

__license__ = "MIT"
__maintainer__ = "Richard Clubb"
__email__ = "richard.clubb@embeduk.com"
__status__ = "Development"


from uds.uds_config_tool.FunctionCreation.iServiceMethodFactory import IServiceMethodFactory
from uds.uds_config_tool.UtilityFunctions import getSdgsDataItem, getSdgsData, getShortName, getLongName, \
                                                 getServiceIdFromDiagService, getParamWithSemantic, \
                                                 getPositiveResponse, getDiagObjectProp, getBitLengthFromDop
from math import ceil

##
# Inputs
# Function name
# expected security response
requestFuncTemplate_getSeed = str("def {0}(suppressResponse=False):\n"
                                  "    securityRequest = {2}\n"
                                  "    if suppressResponse: securityRequest |= 0x80\n"
                                  "    return [{1}, securityRequest]"
                                  )

requestFuncTemplate_sendKey = str("def {0}(key, suppressResponse=False):\n"
                                  "    serviceId = {1}\n"
                                  "    subFunction = {2}\n"
                                  "    if suppressResponse: subFunction |= 0x80\n"
                                  "    return [serviceId, subFunction] + key")

checkSidTemplate = str("def {0}(sid):\n"
                       "    expectedSid = {1}\n"
                       "    if expectedSid != sid: raise Exception(\"SID do not match\")"
                       )

checkSecurityAccessTemplate = str("def {0}(securityAccess):\n"
                                  "    expectedSecurityAccess = {1}\n"
                                  "    if expectedSecurityAccess != securityAccess: raise Exception(\"Security Mode does not match\")"
                                  )

checkReturnedDataTemplate = str("def {0}(data):\n"
                               "    expectedDataLength = {1}\n"
                               "    if expectedDataLength != len(data): raise Exception(\"Returned data length not expected\")"
                               )

checkNegativeResponseTemplate = str("def {0}(data):\n"
                                    "    if data[0] == 0x7F: raise Exception(\"Found negative response\")"
                                    )

##
# inputs:
# Length
checkInputDataTemplate = str("def {0}(data):\n"
                             "    expectedLength = {1}\n"
                             "    if isinstance(data, list):\n"
                             "        if len(data) != expectedLength: raise Exception(\"Input data does not match expected length\")\n"
                             "    else:"
                             "        pass")

class SecurityAccessMethodFactory(object):

    __metaclass__ = IServiceMethodFactory

    ##
    # @brief method to create the request function for the service element
    @staticmethod
    def create_requestFunction(diagServiceElement, xmlElements):
        serviceId = 0x00
        securityRequest = 0x00

        # have to dig out the sgds name for this one
        requestElement = xmlElements[diagServiceElement.find('REQUEST-REF').attrib['ID-REF']]

        sdgsName = getSdgsDataItem(diagServiceElement, "DiagInstanceQualifier")

        serviceId = getServiceIdFromDiagService(diagServiceElement, xmlElements)
        accessMode = getParamWithSemantic(requestElement, "ACCESSMODE")
        subfunction = getParamWithSemantic(requestElement, "SUBFUNCTION")

        # if accessMode is not none then this is a seed request
        if accessMode is not None:
            securityRequest = int(getParamWithSemantic(requestElement, "ACCESSMODE").find("CODED-VALUE").text)
            requestFuncString = requestFuncTemplate_getSeed.format(sdgsName, serviceId, securityRequest)
        elif subfunction is not None:
            securityRequest = int(getParamWithSemantic(requestElement, "SUBFUNCTION").find("CODED-VALUE").text)
            requestFuncString = requestFuncTemplate_sendKey.format(sdgsName, serviceId, securityRequest)
        else:
            requestFuncString = None

        if requestFuncString is not None:
            exec(requestFuncString)
            return locals()[sdgsName]
        else:
            return None

    ##
    # @brief method to create the function to check the positive response for validity
    @staticmethod
    def create_checkPositiveResponseFunction(diagServiceElement, xmlElements):
        responseId = 0
        securityRequest = 0

        responseId = getServiceIdFromDiagService(diagServiceElement, xmlElements) + 0x40
        positiveResponseElement = getPositiveResponse(diagServiceElement, xmlElements)

        diagInstanceQualifier = getSdgsDataItem(diagServiceElement, "DiagInstanceQualifier")

        checkSidFunctionName = "check_{0}_sid".format(diagInstanceQualifier)
        checkSecurityAccessFunctionName = "check_{0}_securityAccess".format(diagInstanceQualifier)
        checkReturnedDataFunctionName = "check_{0}_returnedData".format(diagInstanceQualifier)

        accessmode = getParamWithSemantic(positiveResponseElement, "ACCESSMODE")
        subfunction = getParamWithSemantic(positiveResponseElement, "SUBFUNCTION")

        if accessmode is not None:
            securityRequest = int(accessmode.find("CODED-VALUE").text)
        elif subfunction is not None:
            securityRequest = int(subfunction.find("CODED-VALUE").text)
        else:
            raise Exception("Format not known")

        dataParams = getParamWithSemantic(positiveResponseElement, "DATA")

        if dataParams is not None:
            if isinstance(dataParams, list):
                raise Exception("Currently can not deal with lists of data")
            else:
                dop = getDiagObjectProp(dataParams, xmlElements)
                bitLength = getBitLengthFromDop(dop)
                payloadLength = int(ceil(bitLength / 8))
        else:
            payloadLength = 0

        checkSidFunctionString = checkSidTemplate.format(checkSidFunctionName,
                                                         responseId
                                                         )

        checkSecurityAccessFunctionString = checkSecurityAccessTemplate.format(checkSecurityAccessFunctionName,
                                                                               securityRequest)

        if payloadLength == 0:
            checkReturnedDataString = None
        else:
            checkReturnedDataString = checkReturnedDataTemplate.format(checkReturnedDataFunctionName,
                                                                       payloadLength)
            exec(checkReturnedDataString)

        exec(checkSidFunctionString)
        exec(checkSecurityAccessFunctionString)

        checkSidFunction = locals()[checkSidFunctionName]
        checkSecurityAccessFunction = locals()[checkSecurityAccessFunctionName]

        checkReturnedDataFunction = None
        try:
            checkReturnedDataFunction = locals()[checkReturnedDataFunctionName]
        except:
            pass

        return checkSidFunction, checkSecurityAccessFunction, checkReturnedDataFunction

    ##
    # @brief method to encode the positive response from the raw type to it physical representation
    @staticmethod
    def create_encodePositiveResponseFunction(diagServiceElement, xmlElements):

        raise Exception("Not implemented")

    ##
    # @brief method to create the negative response function for the service element
    @staticmethod
    def create_checkNegativeResponseFunction(diagServiceElement, xmlElements):

        diagInstanceQualifier = getSdgsDataItem(diagServiceElement, "DiagInstanceQualifier")

        checkNegativeResponseFunctionName = "check_{0}_negResponse".format(diagInstanceQualifier)

        checkNegativeResponseFunctionString = checkNegativeResponseTemplate.format(checkNegativeResponseFunctionName)

        exec(checkNegativeResponseFunctionString)

        return locals()[checkNegativeResponseFunctionName]

    @staticmethod
    def check_inputDataFunction(diagServiceElement, xmlElements):

        diagInstanceQualifier = getSdgsDataItem(diagServiceElement, "DiagInstanceQualifier")


if __name__ == "__main__":

    pass
